/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __NTHW_GMF_H__
#define __NTHW_GMF_H__

enum GMF_STATUS_MASK {
  GMF_STATUS_MASK_DATA_UNDERFLOWED = 1,
  GMF_STATUS_MASK_IFG_ADJUSTED
};

struct NthwGmf
{
  nthw_fpga_t* mpFpga;
  nthw_module_t* mpModGmf;
  int mnInstance;

  nthw_register_t* mpCtrl;
  nthw_field_t* mpCtrlEnable;
  nthw_field_t* mpCtrlIfgEnable;
  nthw_field_t* mpCtrlIfgTxNowAlways;
  nthw_field_t* mpCtrlIfgTxOnTsAlways;
  nthw_field_t* mpCtrlIfgTxOnTsAdjustOnSetClock;
  nthw_field_t* mpCtrlIfgAutoAdjustEnable;
  nthw_field_t* mpCtrlTsInjectAlways;
  nthw_field_t* mpCtrlFcsAlways;

  nthw_register_t* mpSpeed;
  nthw_field_t* mpSpeedIfgSpeed;

  nthw_register_t* mpIfgClockDelta;
  nthw_field_t* mpIfgClockDeltaDelta;

  nthw_register_t* mpIfgClockDeltaAdjust;
  nthw_field_t* mpIfgClockDeltaAdjustDelta;

  nthw_register_t* mpIfgMaxAdjustSlack;
  nthw_field_t* mpIfgMaxAdjustSlackSlack;

  nthw_register_t* mpDebugLaneMarker;
  nthw_field_t* mpDebugLaneMarkerCompensation;

  nthw_register_t* mpStatSticky;
  nthw_field_t* mpStatStickyDataUnderflowed;
  nthw_field_t* mpStatStickyIfgAdjusted;

  nthw_register_t* mpStatNextPkt;
  nthw_field_t* mpStatNextPktNs;

  nthw_register_t* mpStatMaxDelayedPkt;
  nthw_field_t* mpStatMaxDelayedPktNs;

  nthw_register_t* mpTsInject;
  nthw_field_t* mpTsInjectOffset;
  nthw_field_t* mpTsInjectPos;
  int mnParamGmfIfgSpeedMul;
  int mnParamGmfIfgSpeedDiv;

  bool mAdministrativeBlock; /* Used to enforce license expiry */
};

typedef struct NthwGmf NthwGmf_t;
typedef struct NthwGmf NthwGmf;

NthwGmf_t* NthwGmf_new(void);
void NthwGmf_delete(NthwGmf_t* p);
int NthwGmf_init(NthwGmf_t* p, nthw_fpga_t* pFpga, int nInstance);
void NthwGmf_setEnableTsi(NthwGmf_t* p, bool enable, int tsiDynamicOffset, int tsiStaticOffset, bool tsiAlways);

void NthwGmf_setEnable(NthwGmf_t* p, bool enable);
void NthwGmf_setIfgEnable(NthwGmf_t* p, bool enable);

void NthwGmf_setTxNowAlwaysEnable(NthwGmf_t* p, bool enable);
void NthwGmf_setTxOnTsAlwaysEnable(NthwGmf_t* p, bool enable);
void NthwGmf_setTxOnTsAdjustOnSetClock(NthwGmf_t* p, bool enable);
void NthwGmf_setIfgAutoAdjustEnable(NthwGmf_t* p, bool enable);

int NthwGmf_getIfgSpeedBitWidth(NthwGmf_t* p);

int NthwGmf_setIfgSpeedRaw(NthwGmf_t* p, uint64_t nSpeedVal);
int NthwGmf_setIfgSpeedBits(NthwGmf_t* p, const uint64_t nRateLimitBits, const uint64_t nLinkSpeed);
int NthwGmf_setIfgSpeedPercent(NthwGmf_t* p, const double fRateLimitPercent);

void NthwGmf_setDelta(NthwGmf_t* p, uint64_t delta);
void NthwGmf_setDeltaAdjust(NthwGmf_t* p, uint64_t deltaAdjust);
void NthwGmf_setSlack(NthwGmf_t* p, uint64_t slack);
void NthwGmf_setCompensation(NthwGmf_t* p, uint32_t compensation);

uint32_t NthwGmf_getStatusSticky(NthwGmf_t* p);
void NthwGmf_setStatusSticky(NthwGmf_t* p, uint32_t status);

uint64_t NthwGmf_getStatNextPktNs(NthwGmf_t* p);
uint64_t NthwGmf_getStatMaxPkDelayedtNs(NthwGmf_t* p);

void NthwGmf_administrativeBlock(NthwGmf_t* p); /* Used to enforce license expiry blocking */

#endif /* __NTHW_GMF_H__ */
