/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __NTHW_IIC_H__
#define __NTHW_IIC_H__

#include "nthw_fpga_model.h"

struct NthwIic
{
  nthw_fpga_t* mpFpga;
  nthw_module_t* mpModIic;
  int mnIicInstance;

  uint32_t mnIicCycleTime;
  int mnPollDelay;
  int mnBusReadyRetry;
  int mnDataReadyRetry;
  int mnReadDataRetry;
  int mnWriteDataRetry;

  nthw_register_t* mpReg_tsusta;
  nthw_field_t* mpFld_tsusta;

  nthw_register_t* mpReg_tsusto;
  nthw_field_t* mpFld_tsusto;

  nthw_register_t* mpReg_thdsta;
  nthw_field_t* mpFld_thdsta;

  nthw_register_t* mpReg_tsudat;
  nthw_field_t* mpFld_tsudat;

  nthw_register_t* mpReg_tbuf;
  nthw_field_t* mpFld_tbuf;

  nthw_register_t* mpReg_thigh;
  nthw_field_t* mpFld_thigh;

  nthw_register_t* mpReg_tlow;
  nthw_field_t* mpFld_tlow;

  nthw_register_t* mpReg_thddat;
  nthw_field_t* mpFld_thddat;

  nthw_register_t* mpReg_cr;
  nthw_field_t* mpFld_cr_en;
  nthw_field_t* mpFld_cr_msms;
  nthw_field_t* mpFld_cr_txfifo_reset;
  nthw_field_t* mpFld_cr_txak;

  nthw_register_t* mpReg_sr;
  nthw_field_t* mpFld_sr_bb;
  nthw_field_t* mpFld_sr_rxfifo_full;
  nthw_field_t* mpFld_sr_rxfifo_empty;
  nthw_field_t* mpFld_sr_txfifo_full;
  nthw_field_t* mpFld_sr_txfifo_empty;

  nthw_register_t* mpReg_tx_fifo;
  nthw_field_t* mpFld_tx_fifo_txdata;
  nthw_field_t* mpFld_tx_fifo_start;
  nthw_field_t* mpFld_tx_fifo_stop;

  nthw_register_t* mpReg_rx_fifo_pirq;
  nthw_field_t* mpFld_rx_fifo_pirq_cmp_val;

  nthw_register_t* mpReg_rx_fifo;
  nthw_field_t* mpFld_rx_fifo_rxdata;

  nthw_register_t* mpReg_softr;
  nthw_field_t* mpFld_softr_rkey;

};

typedef struct NthwIic NthwIic_t;
typedef struct NthwIic NthwIic;

NthwIic_t* NthwIic_new(void);
int NthwIic_init(NthwIic_t *p, nthw_fpga_t* pFpga, int nIicInstance, uint32_t nIicCycleTime);
void NthwIic_delete(NthwIic_t* p);

int NthwIic_SetRetryParams(NthwIic_t* p, const int nPollDelay, const int nBusReadyRetry, const int nDataReadyRetry, const int nReadDataRetry, const int nWriteDataRetry);

int NthwIic_readData(NthwIic_t* p, uint8_t DevAddr, uint8_t RegAddr, uint8_t DataLen, void* pVoid);
int NthwIic_readbyte(NthwIic_t* p, uint8_t DevAddr, uint8_t RegAddr, uint8_t DataLen, uint8_t* pByte);
int NthwIic_writeData(NthwIic_t* p, uint8_t DevAddr, uint8_t RegAddr, uint8_t DataLen, void* pVoid);
int NthwIic_writebyte(NthwIic_t* p, uint8_t DevAddr, uint8_t RegAddr, uint8_t DataLen, uint8_t* pByte);
bool NthwIic_busReady(NthwIic_t* p);
bool NthwIic_dataReady(NthwIic_t* p);

int NthwIic_scan(NthwIic_t* p);
int NthwIic_scan_dev_addr(NthwIic_t* p, int nDevAddr, int nRegAddr);
int NthwIic_scan_find_dev(NthwIic_t* p, int nDevAddrStart, bool bIncreate);

#endif /* __NTHW_IIC_H__ */
