/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __NTHW_MAC_H__
#define __NTHW_MAC_H__

enum NthwMAC_LEDMode {
      LED_AUTO=0x00,
      LED_ON=0x01,
      LED_OFF=0x02,
      LED_PORTID=0x03
};

struct NthwMac {
  nthw_fpga_t* mpFpga;
  nthw_module_t* mpModMac;
  int mnInstance;

  /* LED mode (NT_PORT_CONTROL) */
  nthw_field_t*    mpFldLEDMode;

  /* INT */
  nthw_field_t*    mpFldIntEnable;
  nthw_field_t*    mpFldIntMaxPace;

  /* RX MAC Enable (RX_CONFIG) */
  nthw_field_t*    mpFldTsEop;
  nthw_field_t*    mpFldMinRxFrame;
  nthw_field_t*    mpFldRxEnable;
  nthw_field_t*    mpFldClearHostCounters;
  nthw_field_t*    mpFldClearMacCounters;

  /* RXLOS MOD MASK (RX_CONFIG2) */
  nthw_field_t*    mpFldRxLosMaskLinkEnable;
  nthw_field_t*    mpFldRxLosMaskIntEnable;
  nthw_field_t*    mpFldRxLosMaskRstEnable;

  nthw_field_t*    mpFldModAbsMaskLinkEnable;
  nthw_field_t*    mpFldModAbsMaskIntEnable;
  nthw_field_t*    mpFldModAbsMaskRstEnable;

  /* TFG ADDR */
  nthw_field_t*    mpFldTfgAddrWriteRAMAdr;
  nthw_field_t*    mpFldTfgAddrReadEnable;
  nthw_field_t*    mpFldTfgAddrReadDone;

  /* TFG DATA */
  nthw_field_t*    mpFldTfgDataLength;
  nthw_field_t*    mpFldTfgDataGap;
  nthw_field_t*    mpFldTfgDataId;

  /* TFG CTRL */
  nthw_field_t*    mpFldTfgWrap;
  nthw_field_t*    mpFldTfgRestart;
  nthw_field_t*    mpFldTfgEnable;
  nthw_field_t*    mpFldTfgTimeMode;
  nthw_field_t*    mpFldTfgIdPos;
  nthw_field_t*    mpFldTfgIdEna;
  nthw_field_t*    mpFldTfgTxActive;

  /* TFG REPETITION */
  nthw_field_t*    mpFldTfgRepetitionCount;

  /* RAW ADDR */
  nthw_field_t*    mpFldRawAddrWriteRAMAdr;
  nthw_field_t*    mpFldRawAddrReadEnable;
  nthw_field_t*    mpFldRawAddrReadDone;

  /* RAW DATA */
  nthw_field_t*    mpFldRawData;

  /* RAW CTRL */
  nthw_field_t*    mpFldRawWrap;
  nthw_field_t*    mpFldRawRestart;
  nthw_field_t*    mpFldRawTgEnable;
  nthw_field_t*    mpFldRawOverWrLaneMarkers;
  nthw_field_t*    mpFldRawTgActive;

  /* RAW REPETITION */
  nthw_field_t*    mpFldRawRepetitionCount;

  /* TX_CONFIG */
  nthw_field_t*    mpFldTxHostEnable;
  nthw_field_t*    mpFldTxMacLoopback;

  nthw_field_t*    mpFldTxMacTxConfigPcsRemoteFaultDis;
  nthw_field_t*    mpFldTxMacTxConfigPcsIdleDis;

  /* RX MAC STAT */
  nthw_register_t* mpRegRxMacStatFrames;
  nthw_register_t* mpRegRxMacStatBytes;
  nthw_field_t*    mpFldRxMacStatFrames;
  nthw_field_t*    mpFldRxMacStatBytes;
  nthw_field_t*    mpFldRxMacStatCRC;
  nthw_field_t*    mpFldRxMacStatCV;
  nthw_field_t*    mpTrigger;

  /* LINK SUMMARY */
  nthw_register_t* mpRegLinkSummary;
  nthw_field_t*    mpFldLinkSummaryAbs;
  nthw_field_t*    mpFldLinkSummaryNtPhyLinkState;
  nthw_field_t*    mpFldLinkSummaryLHAbs;
  nthw_field_t*    mpFldLinkSummaryLLPhyLinkState;
  nthw_field_t*    mpFldLinkSummaryLinkDownCnt;
  nthw_field_t*    mpFldLinkSummaryGBoxIntErr;
  nthw_field_t*    mpFldLinkSummaryLHLocalFault;
  nthw_field_t*    mpFldLinkSummaryLHRemoteFault;
  nthw_field_t*    mpFldLinkSummaryLHRxLos;
  nthw_field_t*    mpFldLinkSummaryLHGlbAlarmN;
  nthw_field_t*    mpFldLinkSummaryLocalFault;
  nthw_field_t*    mpFldLinkSummaryRemoteFault;
  nthw_field_t*    mpFldLinkSummaryRxLos;
  nthw_field_t*    mpFldLinkSummaryGlbAlarmN;
};

typedef struct NthwMac NthwMac_t;
typedef struct NthwMac NthwMac;

NthwMac_t* NthwMac_new(void);
void NthwMac_delete(NthwMac_t* p);
int NthwMac_init(NthwMac_t *p, nthw_fpga_t* pFpga, int nInstance);

#if 0
int NthwMAC_getFpgaProductId(NthwMac_t* p);
#endif

void NthwMAC_setLEDMode(NthwMac_t* p, unsigned int mode);
void NthwMAC_rxEnable(NthwMac_t* p, bool enable);
void NthwMAC_clearHostCounters(NthwMac_t* p);
void NthwMAC_clearMacCounters(NthwMac_t* p);

void NthwMAC_setRxLosMaskLink(NthwMac_t* p, bool enable);
void NthwMAC_setRxLosMaskInt(NthwMac_t* p, bool enable);
void NthwMAC_setRxLosMaskRst(NthwMac_t* p, bool enable);

void NthwMAC_setModAbsMaskLink(NthwMac_t* p, bool enable);
void NthwMAC_setModAbsMaskInt(NthwMac_t* p, bool enable);
void NthwMAC_setModAbsMaskRst(NthwMac_t* p, bool enable);

void NthwMAC_TxHostEnable(NthwMac_t* p, bool enable);
void NthwMAC_TxMacLoopbackEnable(NthwMac_t* p, bool enable);
void NthwMAC_TxMacPcsRemoteFaultDis(NthwMac_t* p, bool disable);
void NthwMAC_TxMacPcsIdleDis(NthwMac_t* p, bool disable);

void NthwMAC_TfgTxStart(NthwMac_t* p,
                        uint32_t repetition,
                        uint32_t size,
                        uint32_t pkt_gap);
void NthwMAC_TfgTxStop(NthwMac_t* p);
bool NthwMAC_isTfgTransmitting(NthwMac_t* p);

void NthwMAC_RawTxStart(NthwMac_t* p,
                        uint32_t repetition,
                        uint32_t wrap);
void NthwMAC_RawTxStop(NthwMac_t* p);
bool NthwMAC_isRawTxTransmitting(NthwMac_t* p);
void NthwMAC_RawDataWrite(NthwMac_t* p,
                          uint32_t addr,
                          uint64_t val);

void NthwMAC_readMacCounters(NthwMac_t* p,
                             uint64_t *packets,
                             uint64_t *bytes,
                             uint32_t *cv_errors,
                             uint32_t *fcs_errors);

void NthwMAC_setTsEop(NthwMac_t* p, uint32_t bitEop);
void NthwMAC_setRxMinFrame(NthwMac_t* p, uint32_t min_rx_frame);

void NthwMAC_setIntEnable(NthwMac_t* p, bool enable);
void NthwMAC_setIntMaxPace(NthwMac_t* p, uint32_t max_pace);

void NthwMAC_getLinkSummary(NthwMac_t* p,
                            uint32_t* pModAbs,
                            uint32_t* pLatchHighModAbs,
                            uint32_t* pRxLos,
                            uint32_t* pLatchHighRxLos,
                            uint32_t* pLinkState,
                            uint32_t* pLatchLowLinkState,
                            uint32_t* pLinkDownCnt,
                            uint32_t* pLocalFault,
                            uint32_t* pLatchHighLocalFault,
                            uint32_t* pRemoteFault,
                            uint32_t* pLatchHighRemoteFault,
                            uint32_t* pGlobalAlarm,
                            uint32_t* pLatchHighGlobalAlarm);

#endif /* NTHW_MAC_H_ */
