/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef NTHW_MAC_PCS_H_
#define NTHW_MAC_PCS_H_

enum NthwMacPcs_LEDMode_e {
  NTHW_MAC_PCS_LED_AUTO = 0x00,
  NTHW_MAC_PCS_LED_ON = 0x01,
  NTHW_MAC_PCS_LED_OFF = 0x02,
  NTHW_MAC_PCS_LED_PORTID = 0x03,
};

#define NthwMacPcs_ReceiverModeDFE (0)
#define NthwMacPcs_ReceiverModeLPM (1)

struct NthwMacPcs
{
  uint8_t mPortNo;

  nthw_fpga_t*     mpFpga;
  nthw_module_t*    mpModMacPcs;
  int mnInstance;

  /* Block lock status */
  nthw_field_t*    mpFldBlockLockLock;
  uint32_t        mFldBlockLockLockMask;

  /* Lane lock status */
  nthw_field_t*    mpFldVlDemuxedLock;
  uint32_t        mFldVlDemuxedLockMask;

  /* GTY_STAT */
  nthw_field_t*    mpFldGtyStatRxRstDone0;
  nthw_field_t*    mpFldGtyStatRxRstDone1;
  nthw_field_t*    mpFldGtyStatRxRstDone2;
  nthw_field_t*    mpFldGtyStatRxRstDone3;
  nthw_field_t*    mpFldGtyStatTxRstDone0;
  nthw_field_t*    mpFldGtyStatTxRstDone1;
  nthw_field_t*    mpFldGtyStatTxRstDone2;
  nthw_field_t*    mpFldGtyStatTxRstDone3;
  uint32_t        mFldGtyStatRxRstDoneMask;
  uint32_t        mFldGtyStatTxRstDoneMask;

  /* GTY_LOOP */
  nthw_register_t* mpRegGtyLoop;
  nthw_field_t*    mpFldGtyLoopGtLoop0;
  nthw_field_t*    mpFldGtyLoopGtLoop1;
  nthw_field_t*    mpFldGtyLoopGtLoop2;
  nthw_field_t*    mpFldGtyLoopGtLoop3;

  /* MAC_PCS_CONFIG */
  nthw_field_t*    mpFldPcsConfigTxPathRst;
  nthw_field_t*    mpFldPcsConfigRxPathRst;
  nthw_field_t*    mpFldPcsConfigRxEnable;
  nthw_field_t*    mpFldPcsConfigRxForceResync;
  nthw_field_t*    mpFldPcsConfigRxTestPattern;
  nthw_field_t*    mpFldPcsConfigTxEnable;
  nthw_field_t*    mpFldPcsConfigTxSendIdle;
  nthw_field_t*    mpFldPcsConfigTxSendRfi;
  nthw_field_t*    mpFldPcsConfigTxTestPattern;

  /* STAT PCS */
  nthw_field_t*    mpFldStatPcsRxStatus;
  nthw_field_t*    mpFldStatPcsRxAligned;
  nthw_field_t*    mpFldStatPcsRxAlignedErr;
  nthw_field_t*    mpFldStatPcsRxMisaligned;
  nthw_field_t*    mpFldStatPcsRxInternalLocalFault;
  nthw_field_t*    mpFldStatPcsRxReceivedLocalFault;
  nthw_field_t*    mpFldStatPcsRxLocalFault;
  nthw_field_t*    mpFldStatPcsRxRemoteFault;
  nthw_field_t*    mpFldStatPcsRxHiBer;

  /* STAT_PCS_RX_LATCH */
  nthw_field_t*    mpFldStatPcsRxLatchStatus;

  /* PHYMAC_MISC */
  nthw_field_t*    mpFldPhymacMiscTxSelHost;
  nthw_field_t*    mpFldPhymacMiscTxSelTfg;
  nthw_field_t*    mpFldPhymacMiscTxSelRxLoop;
  nthw_field_t*    mpFldPhymacMiscTsEop;

  /* LINK_SUMMARY */
  nthw_register_t* mpRegLinkSummary;
  nthw_field_t*    mpFldLinkSummaryAbs;
  nthw_field_t*    mpFldLinkSummaryNtPhyLinkState;
  nthw_field_t*    mpFldLinkSummaryLhAbs;
  nthw_field_t*    mpFldLinkSummaryLlNtPhyLinkState;
  nthw_field_t*    mpFldLinkSummaryLinkDownCnt;
  nthw_field_t*    mpFldLinkSummaryNimInterr;
  nthw_field_t*    mpFldLinkSummaryLhLocalFault;
  nthw_field_t*    mpFldLinkSummaryLhRemoteFault;
  nthw_field_t*    mpFldLinkSummaryLocalFault;
  nthw_field_t*    mpFldLinkSummaryRemoteFault;

  /* BIP_ERR */
  nthw_register_t* mpRegBipErr;
  nthw_field_t*    mpFldRegBipErrBipErr;

  /* FEC_CTRL */
  nthw_register_t* mpRegFecCtrl;
  nthw_field_t*    mpFieldFecCtrlRegRsFecCtrlIn;

  /* FEC_STAT */
  nthw_register_t* mpRegFecStat;
  nthw_field_t*    mpFieldFecStatBypass;
  nthw_field_t*    mpFieldFecStatValid;
  nthw_field_t*    mpFieldFecStatAmLock0;
  nthw_field_t*    mpFieldFecStatAmLock1;
  nthw_field_t*    mpFieldFecStatAmLock2;
  nthw_field_t*    mpFieldFecStatAmLock3;
  nthw_field_t*    mpFieldFecStatFecLaneAlgn;

  /* FEC Corrected code word count */
  nthw_register_t* mpRegFecCwCnt;
  nthw_field_t*    mpFieldFecCwCntCwCnt;

  /* FEC Uncorrected code word count */
  nthw_register_t* mpRegFecUcwCnt;
  nthw_field_t*    mpFieldFecUcwCntUcwCnt;

  /* GTY_RX_BUF_STAT */
  nthw_register_t* mpRegGtyRxBufStat;
  nthw_field_t*    mpFieldGtyRxBufStatRxBufStat0;
  nthw_field_t*    mpFieldGtyRxBufStatRxBufStat1;
  nthw_field_t*    mpFieldGtyRxBufStatRxBufStat2;
  nthw_field_t*    mpFieldGtyRxBufStatRxBufStat3;
  nthw_field_t*    mpFieldGtyRxBufStatRxBufStatChanged0;
  nthw_field_t*    mpFieldGtyRxBufStatRxBufStatChanged1;
  nthw_field_t*    mpFieldGtyRxBufStatRxBufStatChanged2;
  nthw_field_t*    mpFieldGtyRxBufStatRxBufStatChanged3;

  /* GTY_PRE_CURSOR */
  nthw_register_t* mpRegGtyPreCursor;
  nthw_field_t*    mpFieldGtyPreCursorTxPreCsr0;
  nthw_field_t*    mpFieldGtyPreCursorTxPreCsr1;
  nthw_field_t*    mpFieldGtyPreCursorTxPreCsr2;
  nthw_field_t*    mpFieldGtyPreCursorTxPreCsr3;

  /* GTY_DIFF_CTL */
  nthw_register_t* mpRegGtyDiffCtl;
  nthw_field_t*    mpFieldGtyGtyDiffCtlTxDiffCtl0;
  nthw_field_t*    mpFieldGtyGtyDiffCtlTxDiffCtl1;
  nthw_field_t*    mpFieldGtyGtyDiffCtlTxDiffCtl2;
  nthw_field_t*    mpFieldGtyGtyDiffCtlTxDiffCtl3;

  /* GTY_POST_CURSOR */
  nthw_register_t* mpRegGtyPostCursor;
  nthw_field_t*    mpFieldGtyPostCursorTxPostCsr0;
  nthw_field_t*    mpFieldGtyPostCursorTxPostCsr1;
  nthw_field_t*    mpFieldGtyPostCursorTxPostCsr2;
  nthw_field_t*    mpFieldGtyPostCursorTxPostCsr3;

  /* GTY_CTL */
  nthw_register_t* mpRegGtyCtl;
  nthw_register_t* mpRegGtyCtlTx;
  nthw_field_t*    mpFieldGtyCtlTxPol0;
  nthw_field_t*    mpFieldGtyCtlTxPol1;
  nthw_field_t*    mpFieldGtyCtlTxPol2;
  nthw_field_t*    mpFieldGtyCtlTxPol3;
  nthw_field_t*    mpFieldGtyCtlRxPol0;
  nthw_field_t*    mpFieldGtyCtlRxPol1;
  nthw_field_t*    mpFieldGtyCtlRxPol2;
  nthw_field_t*    mpFieldGtyCtlRxPol3;
  nthw_field_t*    mpFieldGtyCtlRxLpmEn0;
  nthw_field_t*    mpFieldGtyCtlRxLpmEn1;
  nthw_field_t*    mpFieldGtyCtlRxLpmEn2;
  nthw_field_t*    mpFieldGtyCtlRxLpmEn3;
  nthw_field_t*    mpFieldGtyCtlRxEquaRst0;
  nthw_field_t*    mpFieldGtyCtlRxEquaRst1;
  nthw_field_t*    mpFieldGtyCtlRxEquaRst2;
  nthw_field_t*    mpFieldGtyCtlRxEquaRst3;

  /* DEBOUNCE_CTRL */
  nthw_register_t* mpRegDebounceCtrl;
  nthw_field_t*    mpFieldDebounceCtrlNtPortCtrl;

  /* TIMESTAMP_COMP */
  nthw_register_t* mpRegTimeStampComp;
  nthw_field_t*    mpFieldTimeStampCompRxDly;
  nthw_field_t*    mpFieldTimeStampCompTxDly;

  /* STAT_PCS_RX */
  nthw_register_t* mpRegStatPcsRx;

  /* STAT_PCS_RX */
  nthw_register_t* mpRegStatPcsRxLatch;

  /* PHYMAC_MISC */
  nthw_register_t* mpRegPhymacMisc;

  /* BLOCK_LOCK */
  nthw_register_t* mpRegBlockLock;
};

typedef struct NthwMacPcs NthwMacPcs_t;
typedef struct NthwMacPcs NthwMacPcs;

NthwMacPcs_t* NthwMacPcs_new(void);
int NthwMacPcs_init(NthwMacPcs_t *p, nthw_fpga_t* pFpga, int nInstance);
void NthwMacPcs_delete(NthwMacPcs_t* p);

bool NthwMacPcs_isBlockAndLaneLockLocked(NthwMacPcs_t* p);
bool NthwMacPcs_isGtFsmRxResetDone(NthwMacPcs_t* p);
bool NthwMacPcs_isGtFsmTxResetDone(NthwMacPcs_t* p);
void NthwMacPcs_txPathRst(NthwMacPcs_t* p, bool enable);
void NthwMacPcs_rxPathRst(NthwMacPcs_t* p, bool enable);
bool NthwMacPcs_isRxPathRst(NthwMacPcs_t* p);
void NthwMacPcs_rxForceResync(NthwMacPcs_t* p, bool enable);
void NthwMacPcs_txSendRfi(NthwMacPcs_t* p, bool enable);
bool NthwMacPcs_isDDR3CalibDone(NthwMacPcs_t* p);
void NthwMacPcs_TxHostEnable(NthwMacPcs_t* p, bool enable); /* wrapper - for ease of use */
void NthwMacPcs_setRxEnable(NthwMacPcs_t* p, bool enable);
void NthwMacPcs_setTxEnable(NthwMacPcs_t* p, bool enable);
void NthwMacPcs_setTxSelHost(NthwMacPcs_t* p, bool enable);
void NthwMacPcs_setTxSelTfg(NthwMacPcs_t* p, bool enable);
void NthwMacPcs_setTsEop(NthwMacPcs_t* p, bool enable);
void NthwMacPcs_setHostLoopback(NthwMacPcs_t* p, bool enable);
void NthwMacPcs_setLineLoopback(NthwMacPcs_t* p, bool enable);
void NthwMacPcs_resetBIPCounters(NthwMacPcs_t* p);
void NthwMacPcs_getStatus(NthwMacPcs_t* p, uint8_t *status);
bool NthwMacPcs_getHiBer(NthwMacPcs_t* p);

void NthwMacPcs_getLinkSummary1(NthwMacPcs_t* p,
                    uint32_t* pStatus,
                    uint32_t* pStatusLatch,
                    uint32_t* pAligned,
                    uint32_t* pLocalFault,
                    uint32_t* pRemoteFault);

void NthwMacPcs_getLinkSummary(NthwMacPcs_t* p,
                    uint32_t* pAbs,
                    uint32_t* pNtPhyLinkState,
                    uint32_t* pLhAbs,
                    uint32_t* pLlNtPhyLinkState,
                    uint32_t* pLinkDownCnt,
                    uint32_t* pNimInterr,
                    uint32_t* pLhLocalFault,
                    uint32_t* pLhRemoteFault,
                    uint32_t* pLocalFault,
                    uint32_t* pRemoteFault);


bool NthwMacPcs_resetRequired(NthwMacPcs_t* p);
void NthwMacPcs_setFec(NthwMacPcs_t* p, bool enable);
bool NthwMacPcs_getFecBypass(NthwMacPcs_t* p);
bool NthwMacPcs_getFecValid(NthwMacPcs_t* p);
bool NthwMacPcs_getFecAligned(NthwMacPcs_t* p);
bool NthwMacPcs_getFecStatAnyAmLocked(NthwMacPcs_t* p);
bool NthwMacPcs_getFecStatAllAmLocked(NthwMacPcs_t* p);
void NthwMacPcs_dumpFecStatFields(NthwMacPcs_t* p);
void NthwMacPcs_resetFecCounters(NthwMacPcs_t* p);
bool NthwMacPcs_getGtyRxBufStatError(NthwMacPcs_t* p);
void NthwMacPcs_setGtyTxTuning(NthwMacPcs_t* p, uint8_t lane, uint8_t txPreCsr, uint8_t txDiffCtl, uint8_t txPostCsr);
void NthwMacPcs_swapGtyTxPolarity(NthwMacPcs_t* p, uint8_t lane, bool swap );
void NthwMacPcs_swapGtyRxPolarity(NthwMacPcs_t* p, uint8_t lane, bool swap );
void NthwMacPcs_setReceiverEqualizationMode(NthwMacPcs_t* p, uint8_t mode);
void NthwMacPcs_setLEDMode(NthwMacPcs_t* p, uint8_t mode);
void NthwMacPcs_setTimestampCompRx(NthwMacPcs_t* p, uint16_t rxDly);
void NthwMacPcs_setPortNo(NthwMacPcs_t* p, uint8_t portNo);

uint32_t NthwMacPcs_getFldBlockLockLock(NthwMacPcs_t* p);
uint32_t NthwMacPcs_getFldBlockLockLockMask(NthwMacPcs_t* p);
uint32_t NthwMacPcs_getFldLaneLockLock(NthwMacPcs_t* p);
uint32_t NthwMacPcs_getFldLaneLockLockMask(NthwMacPcs_t* p);

#endif /* NTHW_MAC_PCS_H_ */
