/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef NTHW_MAC_PCS_XXV_H_
#define NTHW_MAC_PCS_XXV_H_

#include <stdint.h>
#include <stdbool.h>
#include "nthw_fpga_model.h"

enum NthwMacPcsXxvLEDMode_e {
  NTHW_MAC_PCS_XXV_LED_AUTO = 0x00,
  NTHW_MAC_PCS_XXV_LED_ON = 0x01,
  NTHW_MAC_PCS_XXV_LED_OFF = 0x02,
  NTHW_MAC_PCS_XXV_LED_PORTID = 0x03,
};

enum NthwMacPcsXxvDACMode_e {
  NTHW_MAC_PCS_XXV_DAC_OFF = 0x00,
  NTHW_MAC_PCS_XXV_DAC_CA_25G_N = 0x01,
  NTHW_MAC_PCS_XXV_DAC_CA_25G_S = 0x02,
  NTHW_MAC_PCS_XXV_DAC_CA_25G_L = 0x03,
};

struct NthwMacPcsXxv {
  nthw_fpga_t* mpFpga;
  nthw_module_t* mpModMacPcsXxv;
  int mnInstance;

  uint8_t mPortNo;

#define NTHW_MAC_PCS_XXV_NUM_ELEMS 4
  struct NthwMacPcsXxv_RegistersFields {
    /* CORE_CONF */
    nthw_register_t* mpRegCoreConf;
    nthw_field_t*    mpFldCoreConfRxEnable;
    nthw_field_t*    mpFldCoreConfRxForceResync;
    nthw_field_t*    mpFldCoreConfTxEnable;
    nthw_field_t*    mpFldCoreConfTxInsFcs;
    nthw_field_t*    mpFldCoreConfTxIgnFcs;
    nthw_field_t*    mpFldCoreConfTxSendLfi;
    nthw_field_t*    mpFldCoreConfTxSendRfi;
    nthw_field_t*    mpFldCoreConfTxSendIdle;
    nthw_field_t*    mpFldCoreConfInlineMode;
    nthw_field_t*    mpFldCoreConfLineLoopback;
    nthw_field_t*    mpFldCoreConfTsAtEop;

    /* ANEG_CONFIG */
    nthw_register_t* mpRegAnegConfig;
    nthw_field_t*    mpFldAnegConfigEnable;
    nthw_field_t*    mpFldAnegConfigBypass;
    nthw_field_t*    mpFldAnegConfigRestart;
    nthw_field_t*    mpFldAnegConfigPseudo;
    nthw_field_t*    mpFldAnegConfigNonceSeed;
    nthw_field_t*    mpFldAnegConfigRemoteFault;
    nthw_field_t*    mpFldAnegConfigPause;
    nthw_field_t*    mpFldAnegConfigAsmdir;
    nthw_field_t*    mpFldAnegConfigFec74Request10G;
    nthw_field_t*    mpFldAnegConfigHideFec74;
    nthw_field_t*    mpFldAnegConfigFec74Request;
    nthw_field_t*    mpFldAnegConfigFec91Request;
    nthw_field_t*    mpFldAnegConfigFec91Ability;
    nthw_field_t*    mpFldAnegConfigRsFecRequest;
    nthw_field_t*    mpFldAnegConfigSwFecOverwrite;
    nthw_field_t*    mpFldAnegConfigSwSpeedOverwrite;

    /* ANEG_ABILITY */
    nthw_register_t* mpRegAnegAbility;
    nthw_field_t*    mpFldAnegAbility25GBaseCR;
    nthw_field_t*    mpFldAnegAbility25GBaseCRS;
    nthw_field_t*    mpFldAnegAbility25GBaseCR1;

    /* LT_CONF */
    nthw_register_t* mpRegLtConf;
    nthw_field_t*    mpFldLtConfEnable;
    nthw_field_t*    mpFldLtConfRestart;
    nthw_field_t*    mpFldLtConfSeed;

    /* SUB_RST */
    nthw_register_t* mpRegSubRst;
    nthw_field_t*    mpFldSubRstRxMacPcs;
    nthw_field_t*    mpFldSubRstTxMacPcs;
    nthw_field_t*    mpFldSubRstRxGtData;
    nthw_field_t*    mpFldSubRstTxGtData;
    nthw_field_t*    mpFldSubRstRxBuf;
    nthw_field_t*    mpFldSubRstRxPma;
    nthw_field_t*    mpFldSubRstTxPma;
    nthw_field_t*    mpFldSubRstRxPcs;
    nthw_field_t*    mpFldSubRstTxPcs;
    nthw_field_t*    mpFldSubRstAnLt;
    nthw_field_t*    mpFldSubRstSpeedCtrl;

    /* SUB_RST_STATUS */
    nthw_register_t* mpRegSubRstStatus;
    nthw_field_t*    mpFldSubRstStatusUserRxRst;
    nthw_field_t*    mpFldSubRstStatusUserTxRst;
    nthw_field_t*    mpFldSubRstStatusQpllLock;

    /* LINK_SUMMARY */
    nthw_register_t* mpRegLinkSummary;
    nthw_field_t*    mpFldLinkSummaryNtPhyLinkState;
    nthw_field_t*    mpFldLinkSummaryLlNtPhyLinkState;
    nthw_field_t*    mpFldLinkSummaryAbs;
    nthw_field_t*    mpFldLinkSummaryLhAbs;
    nthw_field_t*    mpFldLinkSummaryLinkDownCnt;
    /* Begin 2 x 10/25 Gbps only fields: */
    nthw_field_t*    mpFldLinkSummaryLlRxFec74Lock;
    nthw_field_t*    mpFldLinkSummaryLhRxRsfecHiSer;
    nthw_field_t*    mpFldLinkSummaryLlRxRsfecLaneAlignment;
    nthw_field_t*    mpFldLinkSummaryLlTxRsfecLaneAlignment;
    nthw_field_t*    mpFldLinkSummaryLhRxPcsValidCtrlCode;
    /* End 2 x 10/25 Gbps only fields. */
    nthw_field_t*    mpFldLinkSummaryLlRxBlockLock;
    nthw_field_t*    mpFldLinkSummaryLhRxHighBitErrorRate;
    nthw_field_t*    mpFldLinkSummaryLhInternalLocalFault;
    nthw_field_t*    mpFldLinkSummaryLhReceivedLocalFault;
    nthw_field_t*    mpFldLinkSummaryLhLocalFault;
    nthw_field_t*    mpFldLinkSummaryLhRemoteFault;
    nthw_field_t*    mpFldLinkSummaryLhTxLocalFault;
    nthw_field_t*    mpFldLinkSummaryNimInterr;

    /* GTY_LOOP */
    nthw_register_t* mpRegGtyLoop;
    nthw_field_t*    mpFldGtyLoopGtLoop;

    /* GTY_CTL_RX */
    nthw_register_t* mpRegGtyCtlRx;
    nthw_field_t*    mpFldGtyCtlRxPolarity;
    nthw_field_t*    mpFldGtyCtlRxLpmEn;
    nthw_field_t*    mpFldGtyCtlRxEquaRst;

    /* GTY_CTL_TX */
    nthw_register_t* mpRegGtyCtlTx;
    nthw_field_t*    mpFldGtyCtlTxPolarity;
    nthw_field_t*    mpFldGtyCtlTxInhibit;

    /* LINK_SPEED */
    nthw_register_t* mpRegLinkSpeed;
    nthw_field_t*    mpFldLinkSpeed10G;
    nthw_field_t*    mpFldLinkSpeedToggle;

    /* RS_FEC_CONF */
    nthw_register_t* mpRegRsFecConf;
    nthw_field_t*    mpFldRsFecConfRsFecEnable;

    /* DEBOUNCE_CTRL */
    nthw_register_t* mpRegDebounceCtrl;
    nthw_field_t*    mpFieldDebounceCtrlNtPortCtrl;

    /* FEC_CCW_CNT */
    nthw_register_t* mpRegRsFecCcw;
    nthw_field_t*    mpFieldRegRsFecCcwRegRsFecCcwCnt;

    /* FEC_UCW_CNT */
    nthw_register_t* mpRegRsFecUcw;
    nthw_field_t*    mpFieldRegRsFecUcwRegRsFecUcwCnt;

    /* TIMESTAMP_COMP */
    nthw_register_t* mpRegTimestampComp;
    nthw_field_t* mpFieldTimestampCompRxDly;
    nthw_field_t* mpFieldTimestampCompTxDly;

    /* GTY_PRE_CURSOR */
    nthw_register_t* mpRegGtyPreCursor;
    nthw_field_t* mpFieldGtyPreCursorTxPreCsr;

    /* GTY_DIFF_CTL */
    nthw_register_t* mpRegGtyDiffCtl;
    nthw_field_t* mpFieldGtyGtyDiffCtlTxDiffCtl;

    /* GTY_POST_CURSOR */
    nthw_register_t* mpRegGtyPostCursor;
    nthw_field_t* mpFieldGtyPostCursorTxPostCsr;
  } regs[NTHW_MAC_PCS_XXV_NUM_ELEMS];
};

typedef struct NthwMacPcsXxv NthwMacPcsXxv_t;
typedef struct NthwMacPcsXxv NthwMacPcsXxv;

NthwMacPcsXxv_t* NthwMacPcsXxv_new(void);
void NthwMacPcsXxv_delete(NthwMacPcsXxv_t* p);
int NthwMacPcsXxv_init(NthwMacPcsXxv_t* p, nthw_fpga_t* pFpga, int nInstance, int nChannels);

void NthwMacPcsXxv_getLinkSummary(NthwMacPcsXxv_t* p,
                                uint32_t* pAbs,
                                uint32_t* pNtPhyLinkState,
                                uint32_t* pLhAbs,
                                uint32_t* pLlNtPhyLinkState,
                                uint32_t* pLinkDownCnt,
                                uint32_t* pNimInterr,
                                uint32_t* pLhLocalFault,
                                uint32_t* pLhRemoteFault,
                                uint32_t* pLhInternalLocalFault,
                                uint32_t* pLhReceivedLocalFault,
                                uint8_t index
                              );

uint8_t NthwMacPcsXxv_getPortNo(const NthwMacPcsXxv_t *p, uint8_t index);

void NthwMacPcsXxv_setPortNo(NthwMacPcsXxv_t* p, uint8_t portNo);

void NthwMacPcsXxv_setTxEnable(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_setRxEnable(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_rxForceResync(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

void NthwMacPcsXxv_resetRxGtData(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_resetTxGtData(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

void NthwMacPcsXxv_resetAnLt(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_resetSpeedCtrl(NthwMacPcsXxv_t *p, bool enable, uint8_t index);

void NthwMacPcsXxv_setTxSendRfi(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_setTxSendLfi(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_setTxSendLfiTxSendRfi(NthwMacPcsXxv_t* p, bool enableLfi, bool enableRfi, uint8_t index);

bool NthwMacPcsXxv_isDfeEnabled(NthwMacPcsXxv_t* p, uint8_t index);
void NthwMacPcsXxv_setDfe(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

void NthwMacPcsXxv_setRxGtyPolarity(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_setTxGtyPolarity(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

void NthwMacPcsXxv_setTxGtyInhibit(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

void NthwMacPcsXxv_setHostLoopback(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_setLineLoopback(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

bool NthwMacPcsXxv_isUserRxRst(NthwMacPcsXxv_t* p, uint8_t index);
bool NthwMacPcsXxv_isUserTxRst(NthwMacPcsXxv_t* p, uint8_t index);

bool NthwMacPcsXxv_isQpllLock(NthwMacPcsXxv_t* p, uint8_t index);
bool NthwMacPcsXxv_isSubRstReady(NthwMacPcsXxv_t* p, uint8_t index);
bool NthwMacPcsXxv_isAnegEnabled(NthwMacPcsXxv_t* p, uint8_t index);

void NthwMacPcsXxv_setTxSendIdle(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_setTxInsFcs(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

bool NthwMacPcsXxv_getLinkSpeed10G(NthwMacPcsXxv_t* p, uint8_t index);
void NthwMacPcsXxv_setLinkSpeed10G(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_setLinkSpeedToggle(NthwMacPcsXxv_t* p, uint8_t index);

void NthwMacPcsXxv_setRsFecConfRsFecEnable(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

void NthwMacPcsXxv_setLEDMode(NthwMacPcsXxv_t* p, uint8_t mode, uint8_t index);

void NthwMacPcsXxv_setRxMacPcsRst(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_setTxMacPcsRst(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

void NthwMacPcsXxv_resetFecCounters(NthwMacPcsXxv_t* p, uint8_t index);

void NthwMacPcsXxv_setGtyDiff(NthwMacPcsXxv_t* p, uint8_t value, uint8_t index);
void NthwMacPcsXxv_setGtyPre(NthwMacPcsXxv_t* p, uint8_t value, uint8_t index);
void NthwMacPcsXxv_setGtyPost(NthwMacPcsXxv_t* p, uint8_t value, uint8_t index);

void NthwMacPcsXxv_setTsAtEop(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

void NthwMacPcsXxv_setAnegConfigEnable(NthwMacPcsXxv_t *p, bool enable, uint8_t index);
void NthwMacPcsXxv_setAnegConfigBypass(NthwMacPcsXxv_t *p, bool enable, uint8_t index);
void NthwMacPcsXxv_setLtConfEnable(NthwMacPcsXxv_t *p, bool enable, uint8_t index);

void NthwMacPcsXxv_setDACMode(NthwMacPcsXxv_t *p, uint8_t DACMode, uint8_t index);

void NthwMacPcsXxv_setTimestampCompRx(NthwMacPcsXxv_t* p, uint16_t rxDly, uint8_t index);
void NthwMacPcsXxv_setTimestampCompTx(NthwMacPcsXxv_t* p, uint16_t txDly, uint8_t index);

void NthwMacPcsXxv_setAnegConfigFec91Request(NthwMacPcsXxv_t *p, bool enable, uint8_t index);
void NthwMacPcsXxv_setAnegConfigRsFecRequest(NthwMacPcsXxv_t *p, bool enable, uint8_t index);
void NthwMacPcsXxv_setAnegConfigFec74Request(NthwMacPcsXxv_t *p, bool enable, uint8_t index);

bool NthwMacPcsXxv_getLlRxFec74Lock(NthwMacPcsXxv_t* p, uint8_t index);
bool NthwMacPcsXxv_getLlRxRsfecLaneAlignment(NthwMacPcsXxv_t* p, uint8_t index);

#endif /* NTHW_MAC_PCS_XXV_H_ */
