/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef NTHW_PCS_H_
#define NTHW_PCS_H_

struct NthwPcs
{
  nthw_fpga_t* mpFpga;
  nthw_module_t* mpModPcs;
  int mnInstance;

  /* Block lock status */
  nthw_field_t*    mpFldBlockLockLock;
  uint32_t         mFldBlockLockLockMask;

  /* Lane lock status */
  nthw_field_t*    mpFldLaneLockLock;
  uint32_t         mFldLaneLockLockMask;

  /* DDR3 calib status */
  nthw_field_t*    mpFldDdr3CalibDone;

  /* GT RESET FSM */
  nthw_field_t*    mpFldRxRstDone;
  nthw_field_t*    mpFldTxRstDone;
  uint32_t         mFldRxRstDoneMask;
  uint32_t         mFldTxRstDoneMask;

  /* PLL lock status (GTH_CONTROL) */
  nthw_field_t*    mpFldPllLock;
  uint32_t         mFldPllLockMask;

  /* Serdes loopback (GTH_CONFIG) */
  nthw_field_t*    mpFldGtLoop;

  /* clear BIP counters (PCS_CONFIG) */
  nthw_field_t*    mpFldTxRxLoop;
  nthw_field_t*    mpFldLaneAddr;
  nthw_field_t*    mpFldBerRst;
  nthw_field_t*    mpFldBipRst;
  nthw_field_t*    mpFldLaneBlockClr;
  nthw_field_t*    mpFldTimeOffsetRx;

  /* TX tuning for GTH TX transceiver (GTH_TX_TUNING) */
  nthw_field_t*    mpFldGthTxDiffCtrl;
  nthw_field_t*    mpFldGthTxPreCursor;
  nthw_field_t*    mpFldGthTxPostCursor;
  nthw_field_t*    mpFldDrpAddr;

  /* TX and RX polarity swap */
  nthw_field_t*    mpFldPcsTxPol;
  nthw_field_t*    mpFldPcsRxPol;

  /* PCS status register */
  nthw_field_t*    mpFldAlign;
  nthw_field_t*    mpFldHiBer;

  /* PCS BIP COUNT register */
  nthw_field_t*    mpFldBipCount;

  /* GTH SOFT RESET (GTH_CONFIG) */
  nthw_field_t*    mpFldGthSoftReset;

  /* GTH LPM mode enable */
  nthw_field_t*    mpFldGthLpmEna;

  /* GTH RX/TX reset */
  nthw_field_t*    mpFldGthTxPcsRst;
  nthw_field_t*    mpFldGthRxPcsRst;
};

typedef struct NthwPcs NthwPcs_t;
typedef struct NthwPcs NthwPcs;

#define RX_EQUAL_MODE_DFE 0
#define RX_EQUAL_MODE_LPM 1

NthwPcs_t* NthwPcs_new(void);
void NthwPcs_delete(NthwPcs_t* p);
int NthwPcs_init(NthwPcs_t* p, nthw_fpga_t* pFpga, int nInstance);

bool NthwPcs_isBlockAndLaneLockLocked(NthwPcs_t* p);
bool NthwPcs_isPartialBlockLock(NthwPcs_t* p, uint32_t blockLock);
bool NthwPcs_isPartialLaneLock(NthwPcs_t* p, uint32_t laneLock);
bool NthwPcs_isHiBerSet(NthwPcs_t* p);
bool NthwPcs_isGtFsmRxResetDone(NthwPcs_t* p);
bool NthwPcs_isGtFsmTxResetDone(NthwPcs_t* p);
bool NthwPcs_isDDR3CalibDone(NthwPcs_t* p);
bool NthwPcs_isGthPllLocked(NthwPcs_t* p);
void NthwPcs_setSerdesLoopback(NthwPcs_t* p, bool enable);
void NthwPcs_resetBIPCounters(NthwPcs_t* p);
uint16_t NthwPcs_getBipCount(NthwPcs_t* p, uint8_t lane);
void NthwPcs_setGthTxTuning(NthwPcs_t* p, uint8_t  tx_diff_ctrl, uint8_t  tx_pre_cursor, uint8_t  tx_post_cursor );
void NthwPcs_getGthTxTuning(NthwPcs_t* p, uint8_t *tx_diff_ctrl, uint8_t *tx_pre_cursor, uint8_t *tx_post_cursor );
void NthwPcs_swapTxPolarity(NthwPcs_t* p, uint8_t swapMask);
void NthwPcs_swapRxPolarity(NthwPcs_t* p, uint8_t swapMask);
void NthwPcs_setRxEqualizerMode(NthwPcs_t* p, uint8_t mode);
void NthwPcs_getStatus(NthwPcs_t* p, uint8_t *align, uint8_t *hiBer);
void NthwPcs_softResetGth(NthwPcs_t* p);
void NthwPcs_resetRxAndTxGth(NthwPcs_t* p);

uint32_t NthwPcs_getFldBlockLockLock(NthwPcs_t* p);

uint32_t NthwPcs_getFldBlockLockLockMask(NthwPcs_t* p);

uint32_t NthwPcs_getFldLaneLockLock(NthwPcs_t* p);

uint32_t NthwPcs_getFldLaneLockLockMask(NthwPcs_t* p);

void NthwPcs_setTimestampCompRx(NthwPcs_t* p, uint16_t rxDly);

#endif /* NTHW_PCS_H_ */
