/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef NTHW_RPF_HPP_
#define NTHW_RPF_HPP_

#include "nthw_fpga_model.h"
#include "pthread.h"
struct nthw_rpf {
  nthw_fpga_t* mpFpga;

  nthw_module_t* mModRpf;

  int mnInstance;

  nthw_register_t* mpRegControl;
  nthw_field_t* mpFldControlPen;
  nthw_field_t* mpFldControlRppEn;
  nthw_field_t* mpFldControlStTglEn;
  nthw_field_t* mpFldControlKeepAliveEn;

  nthw_register_t* mpTsSortPrg;
  nthw_field_t* mpFldTsSortPrgMaturingDelay;
  nthw_field_t* mpFldTsSortPrgTsAtEof;

  int mDefaultMaturingDelay;
  bool mAdministrativeBlock; /* used to enforce license expiry */

  pthread_mutex_t rpfMutex;
};

typedef struct nthw_rpf nthw_rpf_t;
typedef struct nthw_rpf nt_rpf;

nthw_rpf_t* nthw_rpf_new(void);
void nthw_rpf_delete(nthw_rpf_t* p);
int nthw_rpf_init(nthw_rpf_t* p, nthw_fpga_t* pFpga, int nInstance);
void nthw_rpf_administrative_block(nthw_rpf_t* p);
void nthw_rpf_block(nthw_rpf_t* p);
void nthw_rpf_unblock(nthw_rpf_t* p);
void nthw_rpf_set_maturing_delay(nthw_rpf_t* p, int32_t delay);
int32_t nthw_rpf_get_maturing_delay(nthw_rpf_t* p);
void nthw_rpf_set_ts_at_eof(nthw_rpf_t* p, bool enable);
bool nthw_rpf_get_ts_at_eof(nthw_rpf_t* p);

#endif
