/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef NTHW_SMM_H_
#define NTHW_SMM_H_

#define NTHW_SMM_RX_INSTANCE (0)
#define NTHW_SMM_TX_INSTANCE (1)

typedef struct {
  uint32_t startSeg;
  uint64_t hbSegTablePhyAddr;
  uint32_t hbSegCount;
} hbSegmentSetup_t;

struct NthwSmm
{
  nthw_fpga_t* mpFpga;
  nthw_module_t* mpModSmm;
  int mnInstance;

  int mnInstanceType;

  int mnModuleMajorVersion;
  int mnModuleMinorVersion;

  uint64_t mnModuleVersionPacked64;

  uint32_t nbHb;

  nthw_register_t* mpRegCtrl;
  nthw_field_t* mpFldCtrlEnable;

  nthw_register_t* mpRegReadyStatus;
  nthw_field_t* mpFldReadyStatusD;

  nthw_register_t* mpRegInvalidStickyStatus;
  nthw_field_t* mpFldInvalidStickyStatusD;

  nthw_register_t* mpRegSegMemCtrl;
  nthw_field_t* mpFldSegMemCtrlA;
  nthw_field_t* mpFldSegMemCtrlCnt;

  nthw_register_t* mpRegSegMemData;
  nthw_field_t* mpFldSegMemDataPhyAddr;
  nthw_field_t* mpFldSegMemDataSize;

  nthw_register_t* mpRegStartSegMemCtrl;
  nthw_field_t* mpFldStartSegMemCtrlAdr;
  nthw_field_t* mpFldStartSegMemCtrlCnt;

  nthw_register_t* mpRegStartSegMemData;
  nthw_field_t* mpFldStartSegMemDataSeg;

  uint32_t maHbEnable[4];

  hbSegmentSetup_t maSegmentTableSetup[128];
};

typedef struct NthwSmm NthwSmm_t;
typedef struct Nthwsmm NthwSmm;

NthwSmm_t* NthwSmm_new(void);
void NthwSmm_delete(NthwSmm_t* p);
int NthwSmm_init(NthwSmm_t *p, nthw_fpga_t* pFpga, int nInstance);

void NthwSmm_clear(NthwSmm_t* p);
void NthwSmm_enable(NthwSmm_t* p);
void NthwSmm_disable(NthwSmm_t* p);

int NthwSmm_getSegmentCount(NthwSmm_t* p, uint32_t nHbFeed);
/* Used in inline scenarios to exchange the HB segment setup between RX and TX SMM */
hbSegmentSetup_t* NthwSmm_getSegmentSetup(NthwSmm_t* p, const uint32_t hbNum);
void NthwSmm_setStartSegment(NthwSmm_t* p, uint32_t hbNum, uint32_t startSeg);
void NthwSmm_setSegmentTable(NthwSmm_t* p, uint32_t hbNum, uint64_t hbSegTablePhyAddr, uint32_t hbSegCount);
bool NthwSmm_isHbPhyAdrInvalid(NthwSmm_t* p, uint32_t* invalidHbMask);

/*
 * NOTE: Max. number of host buffers are currently 128 for RX and TX.
 *       If this constant is no longer valid, consider to use dynamic allocation.
 */
void NthwSmm_disableHb(NthwSmm_t* p, uint32_t hbNum);
void NthwSmm_enableHb(NthwSmm_t* p, uint32_t hbNum);
bool NthwSmm_isHbReady(NthwSmm_t* p, uint32_t hbNum);

#endif /* NTHW_SMM_H_ */
