/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef _FLOW_NTHW_CSU_H_
#define _FLOW_NTHW_CSU_H_

#include <stdint.h>
#include "nthw_fpga_model.h"

struct CsuNthw {
  uint8_t mPhysicalAdapterNo;
  nthw_fpga_t *mpFpga;

  nthw_module_t *mCsu;

  nthw_register_t *mpRcpCtrl;
  nthw_field_t *mpRcpCtrlAdr;
  nthw_field_t *mpRcpCtrlCnt;
  nthw_register_t *mpRcpData;
  nthw_field_t *mpRcpDataOl3Cmd;
  nthw_field_t *mpRcpDataOl4Cmd;
  nthw_field_t *mpRcpDataIl3Cmd;
  nthw_field_t *mpRcpDataIl4Cmd;
};

struct CsuNthw *CsuNthw_new(void);
void CsuNthw_delete(struct CsuNthw *p);
int CsuNthw_init(struct CsuNthw *p, nthw_fpga_t *pFpga, int nInstance);

int CsuNthw_setup(struct CsuNthw *p, int nIdx, int nIdxCnt);
void csuNthw_setDebugMode(struct CsuNthw *p, unsigned int nDebugMode);

#if 0
  uint32_t GetVersionMajor(void) {return (mCsu.getVersion()).getMajorVersion();}
  uint32_t GetVersionMinor(void) {return (mCsu.getVersion()).getMinorVersion();}
  void SetDebugOutput(int on) {if (on) mCsu.setDebugMode(Register::ON_WRITE); else mCsu.setDebugMode(Register::NO_DEBUG);}
#endif

void CsuNthw_RcpSelect(const struct CsuNthw *p, uint32_t val);
void CsuNthw_RcpCnt(const struct CsuNthw *p, uint32_t val);
void CsuNthw_RcpOuterL3Cmd(const struct CsuNthw *p, uint32_t val);
void CsuNthw_RcpOuterL4Cmd(const struct CsuNthw *p, uint32_t val);
void CsuNthw_RcpInnerL3Cmd(const struct CsuNthw *p, uint32_t val);
void CsuNthw_RcpInnerL4Cmd(const struct CsuNthw *p, uint32_t val);
void CsuNthw_RcpFlush(const struct CsuNthw *p);

#endif /* _FLOW_NTHW_CSU_H_ */
