/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __FLOW_NTHW_FLM_H__
#define __FLOW_NTHW_FLM_H__

#include <stdint.h> /* uint32_t */
#include "nthw_fpga_model.h"

struct FlmNthw;

typedef struct FlmNthw FlmNthw_t;

struct FlmNthw *FlmNthw_new(void);
void FlmNthw_delete(struct FlmNthw *p);
int FlmNthw_init(struct FlmNthw *p, nthw_fpga_t *pFpga, int nInstance);
#if 0
int FlmNthw_setup(struct FlmNthw *p, int nIdx, int nIdxCnt);
#endif
void flmNthw_setDebugMode(struct FlmNthw *p, unsigned int nDebugMode);

#if 0
uint32_t GetVersionMajor(void) {return (mFlm.getVersion()).getMajorVersion();}
uint32_t GetVersionMinor(void) {return (mFlm.getVersion()).getMinorVersion();}
void SetDebugOutput(int on) {if (on) mFlm.setDebugMode(Register::ON_WRITE); else mFlm.setDebugMode(Register::NO_DEBUG);}
#endif

/* Control */
void FlmNthw_ControlEnable(const struct FlmNthw *p, uint32_t val);
void FlmNthw_ControlInit(const struct FlmNthw *p, uint32_t val);
void FlmNthw_ControlLds(const struct FlmNthw *p, uint32_t val);
void FlmNthw_ControlLfs(const struct FlmNthw *p, uint32_t val);
void FlmNthw_ControlLis(const struct FlmNthw *p, uint32_t val);
void FlmNthw_ControlUds(const struct FlmNthw *p, uint32_t val);
void FlmNthw_ControlUis(const struct FlmNthw *p, uint32_t val);
void FlmNthw_ControlRds(const struct FlmNthw *p, uint32_t val);
void FlmNthw_ControlRis(const struct FlmNthw *p, uint32_t val);
void FlmNthw_ControlPds(const struct FlmNthw *p, uint32_t val);
void FlmNthw_ControlPis(const struct FlmNthw *p, uint32_t val);
void FlmNthw_ControlCrcwr(const struct FlmNthw *p, uint32_t val);
void FlmNthw_ControlCrcrd(const struct FlmNthw *p, uint32_t val);
void FlmNthw_ControlRbl(const struct FlmNthw *p, uint32_t val);
void FlmNthw_ControlEab(const struct FlmNthw *p, uint32_t val);
void FlmNthw_ControlSplitSdramUsage(const struct FlmNthw *p, uint32_t val);
void FlmNthw_ControlFlush(const struct FlmNthw *p);

/* Status */
void FlmNthw_StatusCalibdone(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatusInitdone(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatusIdle(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatusCritical(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatusPanic(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatusCrcerr(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatusEftBp(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatusFlush(const struct FlmNthw *p);
void FlmNthw_StatusUpdate(const struct FlmNthw *p);

/* Timeout */
void FlmNthw_TimeoutT(const struct FlmNthw *p, uint32_t val);
void FlmNthw_TimeoutFlush(const struct FlmNthw *p);

/* Scrub */
void FlmNthw_ScrubI(const struct FlmNthw *p, uint32_t val);
void FlmNthw_ScrubFlush(const struct FlmNthw *p);

/* Load BIN */
void FlmNthw_LoadBin(const struct FlmNthw *p, uint32_t val);
void FlmNthw_LoadBinFlush(const struct FlmNthw *p);

/* Load PPS */
void FlmNthw_LoadPps(const struct FlmNthw *p, uint32_t val);
void FlmNthw_LoadPpsFlush(const struct FlmNthw *p);

/* Load LPS */
void FlmNthw_LoadLps(const struct FlmNthw *p, uint32_t val);
void FlmNthw_LoadLpsFlush(const struct FlmNthw *p);

/* Load APS */
void FlmNthw_LoadAps(const struct FlmNthw *p, uint32_t val);
void FlmNthw_LoadApsFlush(const struct FlmNthw *p);

/* Prio */
void FlmNthw_PrioLimit0(const struct FlmNthw *p, uint32_t val);
void FlmNthw_PrioFt0(const struct FlmNthw *p, uint32_t val);
void FlmNthw_PrioLimit1(const struct FlmNthw *p, uint32_t val);
void FlmNthw_PrioFt1(const struct FlmNthw *p, uint32_t val);
void FlmNthw_PrioLimit2(const struct FlmNthw *p, uint32_t val);
void FlmNthw_PrioFt2(const struct FlmNthw *p, uint32_t val);
void FlmNthw_PrioLimit3(const struct FlmNthw *p, uint32_t val);
void FlmNthw_PrioFt3(const struct FlmNthw *p, uint32_t val);
void FlmNthw_PrioFlush(const struct FlmNthw *p);

/* PST */
void FlmNthw_PstSelect(const struct FlmNthw *p, uint32_t val);
void FlmNthw_PstCnt(const struct FlmNthw *p, uint32_t val);
void FlmNthw_PstBp(const struct FlmNthw *p, uint32_t val);
void FlmNthw_PstPp(const struct FlmNthw *p, uint32_t val);
void FlmNthw_PstTp(const struct FlmNthw *p, uint32_t val);
void FlmNthw_PstFlush(const struct FlmNthw *p);

/* RCP */
void FlmNthw_RcpSelect(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpCnt(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpLookup(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpQw0Dyn(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpQw0Ofs(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpQw0Sel(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpQw4Dyn(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpQw4Ofs(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpSw8Dyn(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpSw8Ofs(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpSw8Sel(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpSw9Dyn(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpSw9Ofs(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpMask(const struct FlmNthw *p, const uint32_t *val);
void FlmNthw_RcpKid(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpOpn(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpIpn(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpBytDyn(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpBytOfs(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpTxplm(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpAutoIpv4Mask(const struct FlmNthw *p, uint32_t val);
void FlmNthw_RcpFlush(const struct FlmNthw *p);

/* Buf Ctrl */
int FlmNthw_BufCtrlUpdate(const struct FlmNthw *p, uint32_t* lrn_free, uint32_t* inf_avail, uint32_t* sta_avail);

/* Lrn Data */
int FlmNthw_LrnDataFlush(const struct FlmNthw *p, const uint32_t *data, uint32_t wordCount, uint32_t* lrn_free, uint32_t* inf_avail, uint32_t* sta_avail);

/* Inf Data */
int FlmNthw_InfDataUpdate(const struct FlmNthw *p, uint32_t *data, uint32_t wordCount, uint32_t* lrn_free, uint32_t* inf_avail, uint32_t* sta_avail);

/* Sta Data */
int FlmNthw_StaDataUpdate(const struct FlmNthw *p, uint32_t *data, uint32_t wordCount, uint32_t* lrn_free, uint32_t* inf_avail, uint32_t* sta_avail);

/* Stat Lrn Done */
void FlmNthw_StatLrnDoneCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatLrnDoneUpdate(const struct FlmNthw *p);

/* Stat Lrn Ignore */
void FlmNthw_StatLrnIgnoreCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatLrnIgnoreUpdate(const struct FlmNthw *p);

/* Stat Lrn Fail */
void FlmNthw_StatLrnFailCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatLrnFailUpdate(const struct FlmNthw *p);

/* Stat Unl Done */
void FlmNthw_StatUnlDoneCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatUnlDoneUpdate(const struct FlmNthw *p);

/* Stat Unl Ignore */
void FlmNthw_StatUnlIgnoreCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatUnlIgnoreUpdate(const struct FlmNthw *p);

/* Stat Prb Done */
void FlmNthw_StatPrbDoneCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatPrbDoneUpdate(const struct FlmNthw *p);

/* Stat Prb Ignore */
void FlmNthw_StatPrbIgnoreCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatPrbIgnoreUpdate(const struct FlmNthw *p);

/* Stat Rel Done */
void FlmNthw_StatRelDoneCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatRelDoneUpdate(const struct FlmNthw *p);

/* Stat Rel Ignore */
void FlmNthw_StatRelIgnoreCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatRelIgnoreUpdate(const struct FlmNthw *p);

/* Stat Aul Done */
void FlmNthw_StatAulDoneCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatAulDoneUpdate(const struct FlmNthw *p);

/* Stat Aul Ignore */
void FlmNthw_StatAulIgnoreCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatAulIgnoreUpdate(const struct FlmNthw *p);

/* Stat Aul Fail */
void FlmNthw_StatAulFailCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatAulFailUpdate(const struct FlmNthw *p);

/* Stat Tul Done */
void FlmNthw_StatTulDoneCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatTulDoneUpdate(const struct FlmNthw *p);

/* Stat Flows */
void FlmNthw_StatFlowsCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatFlowsUpdate(const struct FlmNthw *p);

/* Stat Sta Done */
void FlmNthw_StatStaDoneCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatStaDoneUpdate(const struct FlmNthw *p);

/* Stat Inf Done */
void FlmNthw_StatInfDoneCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatInfDoneUpdate(const struct FlmNthw *p);

/* Stat Inf Skip */
void FlmNthw_StatInfSkipCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatInfSkipUpdate(const struct FlmNthw *p);

/* Stat Pck Hit */
void FlmNthw_StatPckHitCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatPckHitUpdate(const struct FlmNthw *p);

/* Stat Pck Miss */
void FlmNthw_StatPckMissCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatPckMissUpdate(const struct FlmNthw *p);

/* Stat Pck Unh */
void FlmNthw_StatPckUnhCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatPckUnhUpdate(const struct FlmNthw *p);

/* Stat Pck Dis */
void FlmNthw_StatPckDisCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatPckDisUpdate(const struct FlmNthw *p);

/* Stat Csh Hit */
void FlmNthw_StatCshHitCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatCshHitUpdate(const struct FlmNthw *p);

/* Stat Csh Miss */
void FlmNthw_StatCshMissCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatCshMissUpdate(const struct FlmNthw *p);

/* Stat Csh Unh */
void FlmNthw_StatCshUnhCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatCshUnhUpdate(const struct FlmNthw *p);

/* Stat Cuc Start */
void FlmNthw_StatCucStartCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatCucStartUpdate(const struct FlmNthw *p);

/* Stat Cuc Move */
void FlmNthw_StatCucMoveCnt(const struct FlmNthw *p, uint32_t *val, int get);
void FlmNthw_StatCucMoveUpdate(const struct FlmNthw *p);


struct FlmNthw {
  uint8_t mPhysicalAdapterNo;
  nthw_fpga_t *mpFpga;
  void   *mpRac;

  nthw_module_t *mFlm;

  nthw_register_t *mpControl;
  nthw_field_t *mpControlEnable;
  nthw_field_t *mpControlInit;
  nthw_field_t *mpControlLds;
  nthw_field_t *mpControlLfs;
  nthw_field_t *mpControlLis;
  nthw_field_t *mpControlUds;
  nthw_field_t *mpControlUis;
  nthw_field_t *mpControlRds;
  nthw_field_t *mpControlRis;
  nthw_field_t *mpControlPds;
  nthw_field_t *mpControlPis;
  nthw_field_t *mpControlCrcwr;
  nthw_field_t *mpControlCrcrd;
  nthw_field_t *mpControlRbl;
  nthw_field_t *mpControlEab;
  nthw_field_t *mpControlSplitSdramUsage;

  nthw_register_t *mpStatus;
  nthw_field_t *mpStatusCalibdone;
  nthw_field_t *mpStatusInitdone;
  nthw_field_t *mpStatusIdle;
  nthw_field_t *mpStatusCritical;
  nthw_field_t *mpStatusPanic;
  nthw_field_t *mpStatusCrcerr;
  nthw_field_t *mpStatusEftBp;

  nthw_register_t *mpTimeout;
  nthw_field_t *mpTimeoutT;

  nthw_register_t *mpScrub;
  nthw_field_t *mpScrubI;

  nthw_register_t *mpLoadBin;
  nthw_field_t *mpLoadBinBin;

  nthw_register_t *mpLoadPps;
  nthw_field_t *mpLoadPpsPps;

  nthw_register_t *mpLoadLps;
  nthw_field_t *mpLoadLpsLps;

  nthw_register_t *mpLoadAps;
  nthw_field_t *mpLoadApsAps;

  nthw_register_t *mpPrio;
  nthw_field_t *mpPrioLimit0;
  nthw_field_t *mpPrioFt0;
  nthw_field_t *mpPrioLimit1;
  nthw_field_t *mpPrioFt1;
  nthw_field_t *mpPrioLimit2;
  nthw_field_t *mpPrioFt2;
  nthw_field_t *mpPrioLimit3;
  nthw_field_t *mpPrioFt3;

  nthw_register_t *mpPstCtrl;
  nthw_field_t *mpPstCtrlAdr;
  nthw_field_t *mpPstCtrlCnt;
  nthw_register_t *mpPstData;
  nthw_field_t *mpPstDataBp;
  nthw_field_t *mpPstDataPp;
  nthw_field_t *mpPstDataTp;

  nthw_register_t *mpRcpCtrl;
  nthw_field_t *mpRcpCtrlAdr;
  nthw_field_t *mpRcpCtrlCnt;
  nthw_register_t *mpRcpData;
  nthw_field_t *mpRcpDataLookup;
  nthw_field_t *mpRcpDataQw0Dyn;
  nthw_field_t *mpRcpDataQw0Ofs;
  nthw_field_t *mpRcpDataQw0Sel;
  nthw_field_t *mpRcpDataQw4Dyn;
  nthw_field_t *mpRcpDataQw4Ofs;
  nthw_field_t *mpRcpDataSw8Dyn;
  nthw_field_t *mpRcpDataSw8Ofs;
  nthw_field_t *mpRcpDataSw8Sel;
  nthw_field_t *mpRcpDataSw9Dyn;
  nthw_field_t *mpRcpDataSw9Ofs;
  nthw_field_t *mpRcpDataMask;
  nthw_field_t *mpRcpDataKid;
  nthw_field_t *mpRcpDataOpn;
  nthw_field_t *mpRcpDataIpn;
  nthw_field_t *mpRcpDataBytDyn;
  nthw_field_t *mpRcpDataBytOfs;
  nthw_field_t *mpRcpDataTxplm;
  nthw_field_t *mpRcpDataAutoIpv4Mask;

  nthw_register_t *mpBufCtrl;
  nthw_field_t *mpBufCtrlLrnFree;
  nthw_field_t *mpBufCtrlInfAvail;
  nthw_field_t *mpBufCtrlStaAvail;

  nthw_register_t *mpLrnData;
  nthw_register_t *mpInfData;
  nthw_register_t *mpStaData;

  nthw_register_t *mpStatLrnDone;
  nthw_field_t *mpStatLrnDoneCnt;

  nthw_register_t *mpStatLrnIgnore;
  nthw_field_t *mpStatLrnIgnoreCnt;

  nthw_register_t *mpStatLrnFail;
  nthw_field_t *mpStatLrnFailCnt;

  nthw_register_t *mpStatUnlDone;
  nthw_field_t *mpStatUnlDoneCnt;

  nthw_register_t *mpStatUnlIgnore;
  nthw_field_t *mpStatUnlIgnoreCnt;

  nthw_register_t *mpStatPrbDone;
  nthw_field_t *mpStatPrbDoneCnt;

  nthw_register_t *mpStatPrbIgnore;
  nthw_field_t *mpStatPrbIgnoreCnt;

  nthw_register_t *mpStatRelDone;
  nthw_field_t *mpStatRelDoneCnt;

  nthw_register_t *mpStatRelIgnore;
  nthw_field_t *mpStatRelIgnoreCnt;

  nthw_register_t *mpStatAulDone;
  nthw_field_t *mpStatAulDoneCnt;

  nthw_register_t *mpStatAulIgnore;
  nthw_field_t *mpStatAulIgnoreCnt;

  nthw_register_t *mpStatAulFail;
  nthw_field_t *mpStatAulFailCnt;

  nthw_register_t *mpStatTulDone;
  nthw_field_t *mpStatTulDoneCnt;

  nthw_register_t *mpStatFlows;
  nthw_field_t *mpStatFlowsCnt;

  nthw_register_t *mpStatStaDone;
  nthw_field_t *mpStatStaDoneCnt;

  nthw_register_t *mpStatInfDone;
  nthw_field_t *mpStatInfDoneCnt;

  nthw_register_t *mpStatInfSkip;
  nthw_field_t *mpStatInfSkipCnt;

  nthw_register_t *mpStatPckHit;
  nthw_field_t *mpStatPckHitCnt;

  nthw_register_t *mpStatPckMiss;
  nthw_field_t *mpStatPckMissCnt;

  nthw_register_t *mpStatPckUnh;
  nthw_field_t *mpStatPckUnhCnt;

  nthw_register_t *mpStatPckDis;
  nthw_field_t *mpStatPckDisCnt;

  nthw_register_t *mpStatCshHit;
  nthw_field_t *mpStatCshHitCnt;

  nthw_register_t *mpStatCshMiss;
  nthw_field_t *mpStatCshMissCnt;

  nthw_register_t *mpStatCshUnh;
  nthw_field_t *mpStatCshUnhCnt;

  nthw_register_t *mpStatCucStart;
  nthw_field_t *mpStatCucStartCnt;

  nthw_register_t *mpStatCucMove;
  nthw_field_t *mpStatCucMoveCnt;
};

#endif /* __FLOW_NTHW_FLM_H__ */
