/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __FLOW_NTHW_INFO_H__
#define __FLOW_NTHW_INFO_H__

#include <stdint.h> /* uint32_t */
#include "nthw_fpga_model.h"

struct InfoNthw;

struct InfoNthw *InfoNthw_new(void);
void InfoNthw_delete(struct InfoNthw *p);
int InfoNthw_init(struct InfoNthw *p, nthw_fpga_t *pFpga, int nInstance);

int InfoNthw_setup(struct InfoNthw *p, int nIdx, int nIdxCnt);

unsigned int InfoNthw_getNbPhyPorts(const struct InfoNthw *p);
unsigned int InfoNthw_getNbRxPorts(const struct InfoNthw *p);
unsigned int InfoNthw_getLtxAvail(const struct InfoNthw *p);

unsigned int InfoNthw_getNbCategories(const struct InfoNthw *p);
unsigned int InfoNthw_getNbQueues(const struct InfoNthw *p);
unsigned int InfoNthw_getNbCatFuncs(const struct InfoNthw *p);
unsigned int InfoNthw_getNbKMFlowTypes(const struct InfoNthw *p);
unsigned int InfoNthw_getNbPmExt(const struct InfoNthw *p);
unsigned int InfoNthw_getNbLen(const struct InfoNthw *p);
unsigned int InfoNthw_getKccSize(const struct InfoNthw *p);
unsigned int InfoNthw_getKccBanks(const struct InfoNthw *p);
unsigned int InfoNthw_getNbKmCategories(const struct InfoNthw *p);
unsigned int InfoNthw_getNbKmCamBanks(const struct InfoNthw *p);
unsigned int InfoNthw_getNbKmCamRecordWords(const struct InfoNthw *p);
unsigned int InfoNthw_getNbKmCamRecords(const struct InfoNthw *p);
unsigned int InfoNthw_getNbKmTcamBanks(const struct InfoNthw *p);
unsigned int InfoNthw_getNbKmTcamBankWidth(const struct InfoNthw *p);
unsigned int InfoNthw_getNbFlmCategories(const struct InfoNthw *p);
unsigned int InfoNthw_getNbFlmSizeMb(const struct InfoNthw *p);
unsigned int InfoNthw_getNbFlmEntrySize(const struct InfoNthw *p);
unsigned int InfoNthw_getNbFlmVariant(const struct InfoNthw *p);
unsigned int InfoNthw_getNbFlmPrios(const struct InfoNthw *p);
unsigned int InfoNthw_getNbFlmPstProfiles(const struct InfoNthw *p);
unsigned int InfoNthw_getNbHstCategories(const struct InfoNthw *p);
unsigned int InfoNthw_getNbQslCategories(const struct InfoNthw *p);
unsigned int InfoNthw_getNbQslQstEntries(const struct InfoNthw *p);
unsigned int InfoNthw_getNbPdbCategories(const struct InfoNthw *p);
unsigned int InfoNthw_getNbIoaCategories(const struct InfoNthw *p);
unsigned int InfoNthw_getNbRoaCategories(const struct InfoNthw *p);
unsigned int InfoNthw_getNbDbsCategories(const struct InfoNthw *p);
unsigned int InfoNthw_getNbCatKmIfCnt(const struct InfoNthw *p);
unsigned int InfoNthw_getNbCatKmIfM0(const struct InfoNthw *p);
unsigned int InfoNthw_getNbCatKmIfM1(const struct InfoNthw *p);
unsigned int InfoNthw_getNbTpeCategories(const struct InfoNthw *p);
unsigned int InfoNthw_getNbTxCpyWriters(const struct InfoNthw *p);
unsigned int InfoNthw_getNbTxCpyMaskMem(const struct InfoNthw *p);
unsigned int InfoNthw_getNbTxRplDepth(const struct InfoNthw *p);
unsigned int InfoNthw_getNbTxRplExtCategories(const struct InfoNthw *p);
unsigned int InfoNthw_getNbTpeIfrCategories(const struct InfoNthw *p);

struct InfoNthw {
  uint8_t mPhysicalAdapterNo;
  nthw_fpga_t *mpFpga;
  unsigned int nPhyPorts;
  unsigned int nRxPorts;
  unsigned int nLtxAvail;
  unsigned int nbCatFunc;
  unsigned int nbCategories;
  unsigned int nbQueues;
  unsigned int nbFlowTypes;
  unsigned int nbPmExt;
  unsigned int nbLen;
  unsigned int nbKccSize;
  unsigned int nbKccBanks;
  unsigned int nbKmCategories;
  unsigned int nbKmCamBanks;
  unsigned int nbKmCamRecordWords;
  unsigned int nbKmCamRecords;
  unsigned int nbKmTcamBanks;
  unsigned int nbKmTcamBankWidth;
  unsigned int nbFlmCategories;
  unsigned int nbFlmSizeMb;
  unsigned int nbFlmEntrySize;
  unsigned int nbFlmVariant;
  unsigned int nbFlmPrios;
  unsigned int nbFlmPstProfiles;
  unsigned int nbHstCategories;
  unsigned int nbQslCategories;
  unsigned int nbQslQstEntries;
  unsigned int nbPdbCategories;
  unsigned int nbIoaCategories;
  unsigned int nbRoaCategories;
  unsigned int nbDbsCategories;
  unsigned int nbCatKmIfCnt;
  unsigned int mCatKmIfM0;
  unsigned int mCatKmIfM1;
  unsigned int nbTpeCategories;
  unsigned int nbTxCpyWriters;
  unsigned int nbTxCpyMaskMem;
  unsigned int nbTxRplDepth;
  unsigned int nbTxRplExtCategories;
  unsigned int nbTpeIfrCategories;
};

#endif  /* __FLOW_NTHW_INFO_H__ */
