/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __FLOW_NTHW_ROA_H__
#define __FLOW_NTHW_ROA_H__

#include <stdint.h> /* uint32_t */
#include "nthw_fpga_model.h"

struct RoaNthw;

typedef struct RoaNthw RoaNthw_t;
#if 0
typedef struct RoaNthw RoaNthw;
#endif

struct RoaNthw *RoaNthw_new(void);
void RoaNthw_delete(struct RoaNthw *p);
int RoaNthw_init(struct RoaNthw *p, nthw_fpga_t *pFpga, int nInstance);

int RoaNthw_setup(struct RoaNthw *p, int nIdx, int nIdxCnt);
void roaNthw_setDebugMode(struct RoaNthw *p, unsigned int nDebugMode);

#if 0
uint32_t GetVersionMajor(void) {return (mRoa.getVersion()).getMajorVersion();}
uint32_t GetVersionMinor(void) {return (mRoa.getVersion()).getMinorVersion();}
void SetDebugOutput(int on) {if (on) mRoa.setDebugMode(Register_t::ON_WRITE); else mRoa.setDebugMode(Register_t::NO_DEBUG);}
#endif

/* TUN HDR */
void RoaNthw_TunHdrSelect(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunHdrCnt(const struct RoaNthw *p, uint32_t val);

void RoaNthw_TunHdrTunnelHdr(const struct RoaNthw *p, uint32_t *val);
void RoaNthw_TunHdrFlush(const struct RoaNthw *p);

/* TUN CFG */
void RoaNthw_TunCfgSelect(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgCnt(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgTunLen(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgTunType(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgTunVlan(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgIpType(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgIpcsUpd(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgIpcsPrecalc(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgIptlUpd(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgIptlPrecalc(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgVxlanUdpLenUpd(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgTxLagIx(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgRecirculate(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgPushTunnel(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgRecircPort(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgRecircBypass(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgFlush(const struct RoaNthw *p);

/* ROA CONFIG */
void RoaNthw_ConfigFwdRecirculate(const struct RoaNthw *p, uint32_t val);
void RoaNthw_ConfigFwdNormalPcks(const struct RoaNthw *p, uint32_t val);
void RoaNthw_ConfigFwdTxPort0(const struct RoaNthw *p, uint32_t val);
void RoaNthw_ConfigFwdTxPort1(const struct RoaNthw *p, uint32_t val);
void RoaNthw_ConfigFwdCellBuilderPcks(const struct RoaNthw *p, uint32_t val);
void RoaNthw_ConfigFwdNonNormalPcks(const struct RoaNthw *p, uint32_t val);
void RoaNthw_ConfigFlush(const struct RoaNthw *p);

/* LAG */
void RoaNthw_LagCfgSelect(const struct RoaNthw *p, uint32_t val);
void RoaNthw_LagCfgCnt(const struct RoaNthw *p, uint32_t val);
void RoaNthw_LagCfgTxPhyPort(const struct RoaNthw *p, uint32_t val);
void RoaNthw_LagCfgFlush(const struct RoaNthw *p);

struct RoaNthw {
  uint8_t mPhysicalAdapterNo;
  nthw_fpga_t *mpFpga;

  nthw_module_t *mRoa;

  nthw_register_t *mpTunHdrCtrl;
  nthw_field_t *mpTunHdrAddr;
  nthw_field_t *mpTunHdrCnt;
  nthw_register_t *mpTunHdrData;
  nthw_field_t *mpTunHdrDataTunnelHdr;

  nthw_register_t *mpTunCfgCtrl;
  nthw_field_t *mpTunCfgAddr;
  nthw_field_t *mpTunCfgCnt;
  nthw_register_t *mpTunCfgData;
  nthw_field_t *mpTunCfgDataTunLen;
  nthw_field_t *mpTunCfgDataTunType;
  nthw_field_t *mpTunCfgDataTunVlan;
  nthw_field_t *mpTunCfgDataIpType;
  nthw_field_t *mpTunCfgDataIpcsUpd;
  nthw_field_t *mpTunCfgDataIpcsPrecalc;
  nthw_field_t *mpTunCfgDataIptlUpd;
  nthw_field_t *mpTunCfgDataIptlPrecalc;
  nthw_field_t *mpTunCfgDataVxlanUdpLenUpd;
  nthw_field_t *mpTunCfgDataTxLagIx;
  nthw_field_t *mpTunCfgDataRecirculate;
  nthw_field_t *mpTunCfgDataPushTunnel;
  nthw_field_t *mpTunCfgDataRecircPort;
  nthw_field_t *mpTunCfgDataRecircBypass;

  nthw_register_t *mpConfig;
  nthw_field_t *mpConfigFwdRecirculate;
  nthw_field_t *mpConfigFwdNormalPcks;
  nthw_field_t *mpConfigFwdTxPort0;
  nthw_field_t *mpConfigFwdTxPort1;
  nthw_field_t *mpConfigFwdCellBuilderPcks;
  nthw_field_t *mpConfigFwdNonNormalPcks;

  nthw_register_t *mpLagCfgCtrl;
  nthw_field_t *mpLagCfgAddr;
  nthw_field_t *mpLagCfgCnt;
  nthw_register_t *mpLagCfgData;
  nthw_field_t *mpLagCfgDataTxPhyPort;
};

#endif /* __FLOW_NTHW_ROA_H__ */
