/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __NTHW_RAC_H__
#define __NTHW_RAC_H__

#include "nt_util.h"
#include "nthw_bus.h"

struct NthwRac
{
  nthw_fpga_t* mpFpga;
  nthw_module_t* mpModRac;

  pthread_mutex_t mMutex;

  int mnParamRacRabInterfaces;
  int mnParamRacRabObUpdate;

  nthw_register_t* mpRegDummy0;
  nthw_register_t* mpRegDummy1;
  nthw_register_t* mpRegDummy2;

  nthw_register_t* mpRegRabInit;
  nthw_field_t* mpFldRabInit;

  int mnFldRabInitBw;
  uint32_t mnFldRabInitMask;

  nthw_register_t* mpRegDbgCtrl;
  nthw_field_t* mpFldDbgCtrl;

  nthw_register_t* mpRegDbgData;
  nthw_field_t* mpFldDbgData;

  nthw_register_t* mpRegRabIBData;
  nthw_field_t* mpFldRabIBData;

  nthw_register_t* mpRegRabOBData;
  nthw_field_t* mpFldRabOBData;

  nthw_register_t* mpRegRabBufFree;
  nthw_field_t* mpFldRabBufFreeIBFree;
  nthw_field_t* mpFldRabBufFreeIBOvf;
  nthw_field_t* mpFldRabBufFreeOBFree;
  nthw_field_t* mpFldRabBufFreeOBOvf;
  nthw_field_t* mpFldRabBufFreeTimeout;

  nthw_register_t* mpRegRabBufUsed;
  nthw_field_t* mpFldRabBufUsedIBUsed;
  nthw_field_t* mpFldRabBufUsedOBUsed;
  nthw_field_t* mpFldRabBufUsedFlush;

  nthw_register_t* mpRegRabDmaIbLo;
  nthw_field_t* mpFldRabDmaIbLoPhyAddr;

  nthw_register_t* mpRegRabDmaIbHi;
  nthw_field_t* mpFldRabDmaIbHiPhyAddr;

  nthw_register_t* mpRegRabDmaObHi;
  nthw_field_t* mpFldRabDmaObHiPhyAddr;

  nthw_register_t* mpRegRabDmaObLo;
  nthw_field_t* mpFldRabDmaObLoPhyAddr;

  nthw_register_t* mpRegRabDmaIbWr;
  nthw_field_t* mpFldRabDmaIbWrPtr;

  nthw_register_t* mpRegRabDmaIbRd;
  nthw_field_t* mpFldRabDmaIbRdPtr;

  nthw_register_t* mpRegRabDmaObWr;
  nthw_field_t* mpFldRabDmaObWrPtr;

  nthw_register_t* mpRegRabNmbRd;
  nthw_register_t* mpRegRabNmbData;
  nthw_register_t* mpRegRabNmbWr;
  nthw_register_t* mpRegRabNmbStatus;

  uint32_t RAC_RAB_INIT_ADDR;
  uint32_t RAC_RAB_IB_DATA_ADDR;
  uint32_t RAC_RAB_OB_DATA_ADDR;
  uint32_t RAC_RAB_BUF_FREE_ADDR;
  uint32_t RAC_RAB_BUF_USED_ADDR;

  uint32_t RAC_RAB_DMA_IB_LO_ADDR;
  uint32_t RAC_RAB_DMA_IB_HI_ADDR;
  uint32_t RAC_RAB_DMA_OB_LO_ADDR;
  uint32_t RAC_RAB_DMA_OB_HI_ADDR;
  uint32_t RAC_RAB_DMA_IB_RD_ADDR;
  uint32_t RAC_RAB_DMA_OB_WR_ADDR;
  uint32_t RAC_RAB_DMA_IB_WR_ADDR;

  uint32_t RAC_RAB_BUF_FREE_IB_FREE_MASK;
  uint32_t RAC_RAB_BUF_FREE_OB_FREE_MASK;
  uint32_t RAC_RAB_BUF_USED_IB_USED_MASK;
  uint32_t RAC_RAB_BUF_USED_OB_USED_MASK;
  uint32_t RAC_RAB_BUF_USED_FLUSH_MASK;

  uint32_t RAC_RAB_BUF_USED_OB_USED_LOW;

  uint32_t RAC_NMB_RD_ADR_ADDR;
  uint32_t RAC_NMB_DATA_ADDR;
  uint32_t RAC_NMB_WR_ADR_ADDR;
  uint32_t RAC_NMB_STATUS_ADDR;

  bool mDmaActive;

  struct nt_dma_s *mDma;

  volatile uint32_t* mDmaInBuf;
  volatile uint32_t* mDmaOutBuf;

  uint16_t mDmaOutPtrRd;
  uint16_t mDmaInPtrWr;
  uint32_t mInFree;
};

typedef struct NthwRac NthwRac_t;
typedef struct NthwRac NthwRac;

struct DmaBufPtr {
  uint32_t size;
  uint32_t index;
  volatile uint32_t *base;
};

NthwRac_t* NthwRac_new(void);
void NthwRac_delete(NthwRac_t* p);
int NthwRac_init(NthwRac_t *p, nthw_fpga_t* pFpga, struct FpgaInfo_s* pFpgaInfo);

int NthwRac_get_rab_interface_count(const NthwRac_t *p);

int NthwRac_rab_init(NthwRac_t* p, uint32_t rabIntfMask);

int NthwRac_rab_setup(NthwRac_t* p);

int NthwRac_rab_reset(NthwRac_t* p);

int NthwRac_rab_write32(NthwRac_t* p, uint32_t address, RabBusId_t busId, uint32_t wordCnt, const uint32_t *pData);
int NthwRac_rab_write32_dma(NthwRac_t* p, uint32_t address, RabBusId_t busId, uint32_t wordCnt, const uint32_t *pData);
int NthwRac_rab_read32(NthwRac_t* p, uint32_t address, RabBusId_t busId, uint32_t wordCnt, uint32_t* pData);
int NthwRac_rab_read32_dma(NthwRac_t* p, uint32_t address, RabBusId_t busId, uint32_t wordCnt, struct DmaBufPtr* bufPtr);

int NthwRac_rab_flush(NthwRac_t* p);

int NthwRac_rab_dma_begin(NthwRac_t* p);
int NthwRac_rab_dma_commit(NthwRac_t* p);

void NthwRac_regRead32(const struct FpgaInfo_s *pFpgaInfo, uint32_t regAddr, uint32_t *pData);
void NthwRac_regWrite32(const struct FpgaInfo_s *pFpgaInfo, uint32_t regAddr, uint32_t pData);

#endif /* __NTHW_RAC_H__ */
