# 1. Inline UPF demo

The purpose of this section is to outline
- what demo application is and how to run it,
- supported feature set,
- how to run application and which results it generates,
- topology description required for run the application,
- and high-level description of test-cases to be executed.

## 1.1 General Description

Inline UPF demo - it is a DPDK demo application for demonstration of how
Napatech smartNIC deals with RTE flow API to setup HW offloading. The application
simulates the behavior of UPF (User Plane Function) in 5G core network.

Inline UPF demo application looks for port 0 ingress packets containing the
protocols Eth, IP, UDP, GTP, UDP.

When such a packet is received the application will program a flow with
actions to decapsulate the Eth, IP, outer UDP and GTP headers and
encapsulate a new Eth header. Further the packet will be transmitted
on port 1.

A corresponding flow doing the opposite is programmed for port 1
ingress packets.

Once the flows are programmed all packet processing is handled
be HW and packets belonging to the flows are no longer sent to
the application.

Inline UPF demo application supports next feature set:

* `GTP-U` (GPRS Tunneling Protocol user plane) encapsulation and 
decapsulation for IPv4 and IPv6
* `PIR,PBS` RFC2698 Peak Information Rate limit for RX stream
* `RSS` (Receive Side Scaling). 
Allow to distribute RX stream between multiple CPUs and queues. 
* `Pre learning` Allows you to analyze a previously prepared pcap
file and create the corresponding flows before running traffic.
* `NAT` Allows you to check NAT feature and verify how to modify packets by
RTE flow API.

Inline UPF demo is the regular DPDK application that configures
Napatech smartNIC to simulate UPF behavior using the *generic RTE flow API*.
For detailed build and usage instructions of UPF demo application, please
refer to section 2.

The result of the demo application execution is encapsulated and decapsulated
GTPv1U traffic on ports' TX lines. 

To check the metrics and counters you can use NT tools: `monitoring` and
`statistics`.
For detailed counters description please refer to 1.4 section.

1.2 Topology Description

The inline UPF demo setup consists of two Nodes:

* Node 1 - software or hardware traffic generator capable
  to generate traffic with GTPv1U headers

* Node 2 - Server with running inline UPF demo application

>NOTE: Node 2 must have installed Napatech SmartNIC

Nodes are connected between each other.

```
        Node 1                                           Node 2
  +--------------------+                         +----------------------+
  |           |        |       N3 (GTPv1U)       |          |           |
  |  traffic  | port 0 |<----------------------->| port 0   |    UPF    |
  |           | ______ |                         |(RSS, PIR)|   demo    |
  | generator |        |   N6 (without GTPv1)    |          |application|
  |           | port 1 |<----------------------->| port 1   |           |
  |           | ______ |                         |          |           |
  +--------------------+                         +----------------------+
```
__Ports dedications for Node 1__

Port 0 - Uplink traffic   - Transmits packets encapsulated with GTPv1U
Port 1 - Downlink traffic - Transmits packets without GTPv1 headers

__Ports dedications for Node 2__

Port 0 - Uplink traffic (UPF N3)    - 
`Receives` packets encapsulated with GTPv1U and 
`Transmits` packets(from port 1)  encapsulated in GTPv1U
Port 1 - Downlink traffic (UPF N6)  - 
`Receives` packets without GTPv1 headers and 
`Transmits` decapsulated packets(from port 0)

## 1.3 Test cases (From Node 2 side)
The purpose of this section is to provide more detailed description of the
features supported by demo application. 

* 1.3.1. Feature GTPv1U (achieved by raw encap/decap - action for RTE flow)

	This feature allows to encap/decap traffic with GTP headers. 
	This feature is enabled by default and no specific user configuration
	is required to enable/disable it.

  - Port 0 expects packets `encapsulated` with GTPv1U on RX and 
	`decapsulates` them. User can capture `decapsulated` packets on TX port 1.

  - Port 1 expects packets without GTPv1U header on RX and `encapsulate` them. 
	User can capture `encapsulated` packets on TX port 0.

* 1.3.2. Feature PIR,PBS (achieved by MTR - rule for RTE flow)

	This feature allows to set peak information rate for RX stream.

  >NOTE: Traffic policing is implemented in accordance with RFC2698.

* 1.3.3. Feature RSS (achieved by RSS - action for RTE flow)

	This feature allows to distribute traffic among queues.
  User can specify the number of queues by parameter `-r`. By default one queue is used.

  - Packets received and matched on port 0 are distributed among
    specified number of queues (1 by default) on port 0 by `RSS` feature.
  - Packets that are marked as overrated by `MTR` are dropped.
  - User can check RX queue counters using the monitoring tool.

    >NOTE: Packets are distributed among the enabled queues based on
    > IP addresses and UDP ports. Packets belonging to the same flow
    > will always reach the same queue.

* 1.3.4. Feature Pre-Learning
	Parsing prepared pcap file, this feature allows to pre-learn flows in advance,
  before start of the traffic.

* 1.3.5. Meter support
In combination with Pre-learning there is possibility to setup exact number
of meters for each flow. Also there is an option to use unique or common meters for flows.

* 1.3.6. Feature NAT
	Modify packets by HW offloading with RTE flow API.
    - Change IP addresses for inner and outer headers
    - Change source port for TX traffic
    - Automatically create NATing rules
    - Private IP address is fixed
    - Public IP address is sub-network (separate for each queue)
    - Only UDP traffic supported
    - UDP port range is 1024 - 65535
  Can be used with pre-learning.

## 1.4 Statistics

Using the `-s` command line option the application can print statistics for a range of
meter IDs.

To further check counters and metrics user can use `monitoring` and `statistics` tools.
Both of these tools allow to check: RX/TX counters, queues counters and 
flow matcher counters(FLM).
Additionally monitoring tool allows to verify PPS and bitrate.

Please follow build instructions at section 2. below. Please note, that DPDK with NT
patch must be built before NT tools.

>NOTE: NT main application with `ntnic` PMD (e.g. Inline UPF demo) must be up and running,
 before execution of NT tools.

### 1.4.1 How to start monitoring tool

  Please run following commands from the root directory of extracted NT inline package:
  ```bash
  cd tools/ntnic-tools/build/monitoring
  ./monitoring
  ```
<TODO> Please refer to monitoring tool readme file for further usage details.
  
### 1.4.2 How to start statistics tool

  Please run following commands from the root directory of extracted NT inline package:
  ```bash
  cd tools/ntnic-tools/build/statistic
  ```

  - Check TX/RX statistics (Port counters)

  ```bash
  ./statistic -p 0 -g txrx_stats
  ```

  - Check Queue statistics (Queues counters)

  ```bash
  ./statistic -p 0 -g que_stats
  ```

  - Check Color statistics

  ```bash
  ./statistic -p 0 -g color_stats
  ```

  - Check flow matcher statistics

  ```bash
  ./statistic -p 0 -g flm_stats
  ```


# 2. Environment How-to instructions (Node 2)

This section includes detailed instructions on how to build,
configure and run UPF application for Node 2.

## 2.1 Get NT inline release package

  Please download and extract official release package of NT Inline product.
  The inline UPF demo application and NT tools are part of this package as shown below:
  ```
  |-- dpdk_v22.11.1
  |   `-- dpdk_v22.11.1_*_1.0.0.*
  |       |-- examples
  |       |   |-- nt_examples
  |       |   |   |-- forward
  |       |   |   |-- inline_upf_demo
  ...
  `-- tools
      `-- ntnic-tools
  ```
  >NOTE: Exact version and hash number at DPDK directory name is replaced by bash wildcard.

## 2.2 Prepare for build

  Add `'inline_upf_demo'` to the DPDK build system in the `meson.build`.

  Please run following commands from the root directory of extracted NT inline package:

  ```bash
  cd dpdk_v22.11.1/dpdk_v22.11.1_*
  sed -i "s/'vmdq_dcb',/'vmdq_dcb', 'nt_examples\/inline_upf_demo',/" examples/meson.build
  ```

## 2.3 Build DPDK + NTNIC + inline_upf_demo

  In the project root, configure DPDK to be built with debug and examples. 
  Output directory is `build`.

  ```bash
  meson --buildtype=debug -Dexamples=nt_examples/inline_upf_demo -Dnt_relay_core=false build
  ```

  Omit `--buildtype=debug` if debug is not wanted.
  Use `-Dexamples=all` if all dpdk examples shall be built.

  In order to build UPF application, run:

  ```bash
  meson compile -C build
  ```

  or

  ```bash
  ninja -C build
  ```

## 2.4 Build nt_tools to use statistics and monitoring tools

  Please run following commands from the root directory of extracted NT inline package:

  ```bash
  export NT_DPDK_PATH=$PWD/dpdk_v22.11.1/dpdk_v22.11.1_*
  cd tools/ntnic-tools
  meson build
  ninja -C build
  ```

## 2.5 Prerequisites to run

### 2.5.1 Add hugepages

  The reservation of hugepages can be performed at run time.
  This is done by echoing the number of hugepages required
  to a `nr_hugepages` file in the `/sys/kernel/` directory corresponding
  to a specific page size (in Kilobytes). For a single-node system,
  the command to use is as follows
  (assuming that 32 of 1 GB pages are required):

  ```bash
  echo 32 > /sys/kernel/mm/hugepages/hugepages-1048576kB/nr_hugepages
  ```

  To make the hugepages of size 1 GB available for DPDK use,
  following steps must be performed:

  ```bash
  mkdir /mnt/huge
  mount -t hugetlbfs pagesize=1GB /mnt/huge
  ```

### 2.5.2 Install supported FPGA

  Please install FPGA image delivered with NT inline package. The image is available at `images` directory.
  The inline UPF demo application won't work correctly if unsupported FPGA image version is used.

### 2.5.3 Load the vfio-pci driver with the command

  ```bash
  modprobe vfio-pci enable_sriov=1
  ```

### 2.5.4 Bind device to DPDK

  Find the PCI id for the Napatech adapter by running the following command
   from the DPDK root:

  ```bash
  ./usertools/dpdk-devbind.py  --status
  ```

  ```log
  Other Network devices
  =====================
  0000:10:00.0 'NT200A02 Network Adapter 01c5' unused=vfio-pci
  ```

  In our case we want to use the adapter `NT200A02 Network Adapter 01c5`.
  The PCI id is `0000:10:00.0`.

  Use the PCI id to bind the vfio-pci driver to the adapter
  using the following command:

  ```bash
  ./usertools/dpdk-devbind.py -b vfio-pci 0000:10:00.0
  ```

## 2.6 How to start inline UPF demo application

Inline UPF demo runs as common DPDK application with only one
mandatory argument (allowed PCI id)

```bash
cd dpdk_v22.11.1/dpdk_v22.11.1_*/build/examples
./dpdk-inline_upf_demo
```

Syntax:
```bash
./dpdk-inline_upf_demo -a <PCI addr>
  [-- {[-p <port number>,pir=<num>{M|G}[,pbs=<num>{M|G}][,mtu=<num>]] [-h] [-v] [-f <filename>] }
  [-m <count_of_meters_per_flow> [-u] [-s <start_MTR_id-end_MTR_id>]] -r {<comma_separated_queue_IDs>|all}]
```

Use `-a` option to specify adapter`s PCI address. It is mandatory.

### 2.6.1 Application also contains internal CLI arguments

#### 2.6.1.1 Port configuration
Use `-p` command to config port-related features like PIR,PBS.

Syntax:
```bash
-p <port_num>[,pir=<num>{M | G}[,pbs=<num>{M|G}][,mtu=<num>]]
```
* `port_num` - valid port number of supported inserted NIC, valid port numbers are 0 and 1.
* `pir` - Peak Information Rate in bytes/s. (This configuration is acceptable only for port 0)
  * `num` - number greater than zero in bytes per second
  * `M` - Mega multiplier
  * `G` - Giga multiplier
* `pbs` - Peak Burst Size in bytes.
  * `num` - number greater than zero in bytes
  * `M` - Mega multiplier
  * `G` - Giga multiplier
* `mtu` - Max MTU in bytes(default value 1500).

Usage examples:
```bash
./dpdk-inline_upf_demo -m 13312  -- -p 0,pir=100M
./dpdk-inline_upf_demo -m 13312  -- -p 0,pir=1G,pbs=100000
./dpdk-inline_upf_demo -m 13312  -- -p 0,pir=1G,pbs=100000,mtu=2000
./dpdk-inline_upf_demo -m 13312  -- -p 0,mtu=2000 -p 1,mtu=2000
```

#### 2.6.1.2 Verbose logging
Use `-v` command to display verbose logs of packets reading and learning.

Syntax:
```bash
-v
```

Usage example:
```bash
./dpdk-inline_upf_demo -m 13312  -- -v
```

#### 2.6.1.3 Pre Learning feature
Use `-f` command to specify pcap file for Pre Learning feature

Syntax:
```bash
-f <file.pcap>
```

Usage example:
```bash
./dpdk-inline_upf_demo -m 13312 -- -f file.pcap
```

#### 2.6.1.4 Policy supporting
This command works only in combination with Pre-Learning.

* Use `-m` command to specify number of common policies per flow.
  Supported values are `1` or `2`, it means that 1 or 2 common policies
  will be applied per each flow.
  Syntax:
  ```bash
  -m <number>
  ```

* Use `-u` command with `-m` to enable unique policy per flow.
  Supported values are `1` or `2`, it means that 1 or 2 unique policies
  will be applied per each flow.
  Syntax:
  ```bash
  -m <number> -u
  ```

* Use extra option `,pir=<rate>[,pbs=<bytes>]` right after number of policies to set
  Peak Information Rate and Peak BURST Size. Rate must be set in **bytes** per second.
  The configuration supports suffixes `G` and `M` to set rate directly in
  Gigabits and Megabits per second respectively.

  Syntax:
  ```bash
  -m <number>[,pir={number}{M|G}[,pbs={number}{M|G}]]
  ```

>NOTE: The default PIR value is 12.500.000.000 bytes/s

>NOTE: The default PBS value is 4.000.000 bytes



Usage examples:

1) Create one common policy and apply it to each flow
  ```bash
  ./dpdk-inline_upf_demo  -m 13312 -- -f ./test.pcap -m 1
  ```

2) Create one unique policy for each flow
  ```bash
  ./dpdk-inline_upf_demo  -m 13312 -- -f ./test.pcap -m 1 -u
  ```

3) Create one common policy with PIR value 10 GBps and apply it to each flow
  ```bash
  ./dpdk-inline_upf_demo  -m 13312 -- -f ./test.pcap -m 1,pir=10G
  ```
4) Create one unique policy with PIR value 10 GBps for each flow
  ```bash
  ./dpdk-inline_upf_demo  -m 13312 -- -f ./test.pcap -m 1,pir=10G -u
  ```

5) Create two common policies with PIR values 10 GBps and 5 GBps
and apply them to each flow
  ```bash
  ./dpdk-inline_upf_demo  -m 13312 -- -f ./test.pcap -m 2,pir=10G,pir=5G
  ```

6) Create two unique policies with PIR values 10 GBps and 5 GBps for each flow
  ```bash
  ./dpdk-inline_upf_demo  -m 13312 -- -f ./test.pcap -m 2,pir=10G,pbs=5000000,pir=5G -u
  ```
7) Create two unique policies with PIR values 10 GBps and 100 GBps for each flow
  ```bash
  ./dpdk-inline_upf_demo  -m 13312 -- -f ./test.pcap -m 2,pir=10G,pbs=9000000 -u
  ```

#### 2.6.1.5 Help menu
Use `-h` command to display help menu with information, mentioned above

Syntax:
```bash
-h
```

Usage example:
```bash
./dpdk-inline_upf_demo -- -h
```
If other internal options are specified along with this parameter, they are ignored.

#### 2.6.1.6 Feature NAT

NAT is disabled by default.
Use option `-n` without parameters to enable feature NAT

Syntax:
```bash
-n
```

Usage example:
```bash
./dpdk-inline_upf_demo  -m 13312 -- -n
```

#### 2.6.1.7 Feature RSS

RSS is disabled by default.
Use option `-r` to specify queues IDs for RSS feature

Syntax:
```bash
-r {<comma_separated_queue_IDs>|all}
```

Usage examples:
```bash
./dpdk-inline_upf_demo -m 13312  -a 0000:10:00.0,rxqs=4 -- -r 1,3
./dpdk-inline_upf_demo -m 13312  -a 0000:10:00.0,rxqs=4 -- -r all
```

#### 2.6.1.8

Use option `-s` to specify a range of MTR IDs for statistics gathering
>NOTE 1: This option *shall* be used with parameters `-m <policies_per_flows> -u` to have unique MTR for each flow.
 In case that parameters `-m <policies_per_flows> -u` are omitted, then statistics for meter ID 0 will be shown.
 In case that parameter `-u` isn't used, then statistics for meter ID 1 or 2 will be shown, based on value of `policies_per_flows`.

>NOTE 2: Memory must be pre-allocated with EAL option `-m` (13G minium. e.g. `-m 13312`)

>NOTE 3: MTR IDs 0, 1 and 2 are reserved for shared PIR,PBS configuration


Syntax:
```bash
-s <start_meter>-<end_meter>
```

Usage example:
Display statistics for MTRs with IDs 1-100
```bash
./dpdk-inline_upf_demo -a 0000:10:00.0 -m 13312 -- -m 1 -u -s 1-100
```

# 3. Traffic generator How-to instructions (Node 1)

This section includes guidelines on how to prepare
Node 1 to work with traffic generator tools.

There are 2 supported options on how to send traffic to demo application:
- re-play prepared PCAP file;
- use TREX traffic generator.

Note: Preparation and Usage of traffic generator tools require 
'Napatech Link-Capture™' Software and the 'Napatech libpcap'
to be installed.

Detailed instructions on how to install necessary software:
https://docs.napatech.com/r/Link-CaptureTM-Software-Getting-Started-Guide/Installing-Napatech-Link-CaptureTM-Software

Detailed instructions on how to run traffic generator tools: [./traffic_gen_tools_conf/README.md](./traffic_gen_tools_conf/README.md).

# 4. Traffic Flows Configuration
This section includes instructions on how to prepare traffic for inline UPF demo.
This section provides abstract high level traffic structure description without
any specific instructions for concrete tools.
It should be applied to selected traffic generator(NT TG, Xena, Trex, etc.),
taking into account specific TG configuration capabilities. 

Additionally, this section contains test cases description (required to check main
demo application features) updated with traffic flows.

## 4.1 Flow structure for traffic generator to achieve flow match ( From Node 1 side )

A traffic generator on Node 1 generates traffic with next structure

Packets encapsulated with GTPv1U. Must be transmitted from port 0 Node 1 (Traffic generator) and have next structure

```scapy
Eth / IPv4 / UDP / GTP_U_Header / IPv4 / UDP
```

or

```scapy
Eth / IPv6 / UDP / GTP_U_Header / IPv6 / UDP
```

Packets  without GTPv1U. Must be transmitted from port 1 Node 1 (Traffic generator) and have next structure

```scapy
Eth / IPv4 / UDP
```

or

```scapy
Eth / IPv6 / UDP
```

## 4.2 Test cases with headers values

### 4.2.1 GTPv1 fields packet

The GTPv1U header values are common for all test cases

__GTPv1u fields__

|                             |            |                                     |
|-----------------------------|------------|-------------------------------------|
| Version                     | 0x1        |    GTP release 99  v1               |
| Protocol type               | 0x1        |    GTP version 1                    |
| Next Extension header flag  | 0x1        |    Next Extension header presented  |
| Sequence number flag        | 0x0        |    Sequence number not presented    |
| N-PDU number flag           | 0x0        |    N-PDU number not presented       |
| msg type                    | 0xff       |    T-PDU                            |
| teid                        |            |    can be configured any value      |
| Next Extension header       |            |    PDU container                    |
| PDU type                    | 0x1        |    UL PDU session information       |
| QFI                         | 0x1        |    QoS flow identifier              |
| Next Extension header       | 0x0        |    No more extension heater         |

### 4.2.2 Uplink traffic

```
Traffic Generator (GTPv1U) ----> port 0 (RSS, PIR) | Inline UPF demo | port 1 ----> decapsulated traffic
```

#### 4.2.2.1. IPv4 Uplink test case

Packet with size 925 bytes transmitted from port 0 Node 1
and received on port 1 Node 2

|          |         Ethernet      |          IPv4                |   UDP  |      GTPv1U     |        inner   IPv4           |   inner UDP     |
|----------|-----------------------|------------------------------|--------|-----------------|-------------------------------|-----------------|
| SRC      |   16:16:16:16:16:16   | 20.0.0.0                     | 2152   |  Look in table  |    20.0.0.0                   |      4046       |
| DST      |   1a:1a:1a:1a:1a:1a   | 21.0.0.0 - 255.255.255.255   | 2152   |  GTPv1u fields  |    21.0.0.0 - 255.255.255.255 |      4097       |

Expected result is decapsulated packets without GTPv1 header transmitted from port 1 Node 2

|          |         Ethernet      |        inner   IPv4           |   inner UDP     |
|----------|-----------------------|-------------------------------|-----------------|
| SRC      |   17:17:17:17:17:17   |    20.0.0.0                   |      4096       |
| DST      |   18:18:18:18:18:18   |    21.0.0.0 - 255.255.255.255 |      4097       |

#### 4.2.2.2. IPv6 Uplink test case

Packet with size 925 bytes transmitted from port 0 Node 1
and received on port 1 Node 2

|          |         Ethernet      |                    IPv6                 |  UDP   |      GTPv1U     |        inner   IPv6                |   inner UDP     |
|----------|-----------------------|-----------------------------------------|--------|-----------------|------------------------------------|-----------------|
| SRC      |   16:16:16:16:16:16   | 5555::  -  5555::00ff                   | any    |   Look in table |        5555::  -  5555::00ff       |     4096        |
| DST      |   1a:1a:1a:1a:1a:1a   | aaaa::                                  | 2152   |  GTPv1u fields  |        aaaa::                      |     4097        |

>IPv6 = 5555:0000:0000:0000:0000:0000:0000:0000 = 5555:: in short format

>IPv6 = aaaa:0000:0000:0000:0000:0000:0000:0000 = aaaa:: in short format

Expected result is decapsulated packets without GTPv1 header transmitted from port 1 Node 2

|          |       Ethernet        |        inner   IPv6                |   inner UDP     |
|----------|-----------------------|------------------------------------|-----------------|
| SRC      |   17:17:17:17:17:17   |        5555::  -  5555::00ff       |     4096        |
| DST      |   18:18:18:18:18:18   |        aaaa::                      |     4097        |

#### 4.2.2.3. IPv4 Uplink test case with NAT

  N3 interface address - 10.10.10.3
  N6 interface sub-networks:

- 20.20.0.0/16 (For RSS queue id 0 )
- 20.21.0.0/16 (For RSS queue id 1 )
- 20.22.0.0/16 (For RSS queue id 2 )
- 20.23.0.0/16 (For RSS queue id 3 )

NAT port range (1024 - 65535)

  Packet with size 925 bytes transmitted from port 0 Node 1
  and received on port 1 Node 2

  |          |         Ethernet      |          IPv4                |   UDP  |      GTPv1U     |        inner   IPv4               |   inner UDP     |
  |----------|-----------------------|------------------------------|--------|-----------------|-----------------------------------|-----------------|
  | SRC      |   16:16:16:16:16:16   | 10.10.10.2 - 10.10.10.63     | 2152   |  Look in table  |    192.168.0.2 - 255.255.255.255  |      4046       |
  | DST      |   1a:1a:1a:1a:1a:1a   | 10.10.10.3                   | 2152   |  GTPv1u fields  |    1.1.1.2                        |      4097       |

  Expected result is decapsulated packets without GTPv1 header transmitted from port 1 Node 2

  |          |         Ethernet      |        inner   IPv4           |           inner UDP             |
  |----------|-----------------------|-------------------------------|---------------------------------|
  | SRC      |   18:18:18:18:18:18   |    20.20.0.0                  |      NAT port(1024-65535)       |
  | DST      |   17:17:17:17:17:17   |    1.1.1.2                    |      4097                       |

#### 4.2.2.4. IPv4 Uplink test case with NAT + pre-learning

  N3 interface address - 10.10.10.3
  N6 interface sub-networks:

- 20.20.0.0/16 (For RSS queue id 0 )
- 20.21.0.0/16 (For RSS queue id 1 )
- 20.22.0.0/16 (For RSS queue id 2 )
- 20.23.0.0/16 (For RSS queue id 3 )

NAT port range (1024 - 65535)

__How to use inline UPF demo with NAT and pre-learing__

  1. Generate a PCAP file that include packets with further structure

  |          |         Ethernet      |          IPv4                |   UDP  |      GTPv1U     |        inner   IPv4               |   inner UDP     |
  |----------|-----------------------|------------------------------|--------|-----------------|-----------------------------------|-----------------|
  | SRC      |   16:16:16:16:16:16   | 10.10.10.2 - 10.10.10.63     | 2152   |  Look in table  |    192.168.0.2 - 255.255.255.255  |      4046       |
  | DST      |   1a:1a:1a:1a:1a:1a   | 10.10.10.3                   | 2152   |  GTPv1u fields  |    1.1.1.2                        |      4097       |
  |          |                       |                              |        |  teid range:    |                                   |                 |
  |          |                       |                              |        | 1 - 1M/10M/100M |                                   |                 |

  >NOTE: The ranges of fields are circular

  >HINT: Better to keep timestamps with zero values.
  Use tool `editcap` to reset timestamps to zero.
  E.g. `editcap -S -0  generated.pcap zero_ts.pcap`

  2. Replay this PCAP file with any replay tool to **port 0 Node 2**

  ```bash
  /opt/natatech3/bin/pktgen -f PCAP_NAT/NAT_ul_1M_128b.pcap -p 0 -n 0 -g 0
  ```

  Expected result is decapsulated packets without GTPv1 header transmitted from port 1 Node 2

  |          |         Ethernet      |        inner   IPv4           |           inner UDP             |
  |----------|-----------------------|-------------------------------|---------------------------------|
  | SRC      |   18:18:18:18:18:18   |    20.20.0.0                  |      NAT port(1024-65535)       |
  | DST      |   17:17:17:17:17:17   |    1.1.1.2                    |      4097                       |


#### 4.2.2.5. IPv6 Uplink test case with NAT

  *IPv6 NAT is not supported yet*

### 4.2.3 Downlink traffic

```
Traffic Generator ---->  port 1 | Inline UPF demo | port 0 ----> encapsulated traffic
```

#### 4.2.3.1. IPv4 Downlink test case

  Packet with size 925 bytes transmitted from port 1 Node 1 and received on port 0 Node 2

  |          |         Ethernet      |        inner   IPv4           |   inner UDP     |
  |----------|-----------------------|-------------------------------|-----------------|
  | SRC      |   18:18:18:18:18:18   |    20.0.0.0                   |      4096       |
  | DST      |   17:17:17:17:17:17   |    21.0.0.0 - 255.255.255.255 |      4097       |

Expected result is encapsulated packets with GTPv1 header transmitted from port 0 Node 2

  |          |         Ethernet      |          IPv4           |   UDP  |      GTPv1U     |        inner   IPv4      |   inner UDP     |
  |----------|-----------------------|-------------------------|--------|-----------------|--------------------------|-----------------|
  | SRC      |   16:16:16:16:16:16   | 10.10.10.3              | 2152   |  Look in table  |    20.0.0.0              |      4096       |
  | DST      |   1a:1a:1a:1a:1a:1a   | 10.10.10.2              | 2152   |  GTPv1u fields  |    21.0.0.0 - 21.0.0.255 |      4097       |

#### 4.2.3.2. IPv6 Downlink test case

  Packet with size 925 bytes transmitted from port 1 Node 1 and received on port 0 Node 2

  |          |       Ethernet        |        inner   IPv6                |   inner UDP     |
  |----------|-----------------------|------------------------------------|-----------------|
  | SRC      |   18:18:18:18:18:18   |        5555::  -  5555::00ff       |     4096        |
  | DST      |   17:17:17:17:17:17   |        aaaa::                      |     4097        |

  Expected result is encapsulated packets with GTPv1 header transmitted from port 0 Node 2

  |          |         Ethernet      |                    IPv6                 |  UDP   |      GTPv1U     |        inner   IPv6                |   inner UDP     |
  |----------|-----------------------|-----------------------------------------|--------|-----------------|------------------------------------|-----------------|
  | SRC      |   16:16:16:16:16:16   |               AAAA::BBBB                | 2152   |   Look table    |        5555::  -  5555::00ff       |     4096        |
  | DST      |   1a:1a:1a:1a:1a:1a   |               1111::2222                | 2152   |  GTPv1u fields  |        aaaa::                      |     4097        |

#### 4.2.3.3. IPv4 Downlink test case with NAT

N3 interface address - 10.10.10.3

N6 interface sub-networks:
- 20.20.0.0/16 (For RSS queue id 0 )
- 20.21.0.0/16 (For RSS queue id 1 )
- 20.22.0.0/16 (For RSS queue id 2 )
- 20.23.0.0/16 (For RSS queue id 3 )

NAT port range (1024 - 65535)

Packet with size 925 bytes transmitted from port 0 Node 1
  and received on port 1 Node 2

  |          |         Ethernet      |          IPv4           |            UDP         |
  |----------|-----------------------|-------------------------|------------------------|
  | SRC      |   17:17:17:17:17:17   | 1.1.1.2                 |          4097          |
  | DST      |   18:18:18:18:18:18   | 20.20.0.0               |  NAT port(1024-65535)  |

Expected result is encapsulated packets with GTPv1 header transmitted from port 0 Node 2

  |          |         Ethernet      |          IPv4                |   UDP  |      GTPv1U     |        inner   IPv4               |   inner UDP     |
  |----------|-----------------------|------------------------------|--------|-----------------|-----------------------------------|-----------------|
  | SRC      |   16:16:16:16:16:16   | 10.10.10.2                   | 2152   |  Look in table  |    192.168.0.2 - 255.255.255.255  |      4046       |
  | DST      |   1a:1a:1a:1a:1a:1a   | 10.10.10.3                   | 2152   |  GTPv1u fields  |    1.1.1.2                        |      4097       |

#### 4.2.3.4. IPv4 Downlink test case with NAT + pre-learning

N3 interface address - 10.10.10.3

N6 interface sub-networks:
- 20.20.0.0/16 (For RSS queue id 0 )
- 20.21.0.0/16 (For RSS queue id 1 )
- 20.22.0.0/16 (For RSS queue id 2 )
- 20.23.0.0/16 (For RSS queue id 3 )

NAT port range (1024 - 65535)

__How to use inline UPF demo with NAT and pre-learning__

  1. Generate a PCAP file that include packets with further structure

  |          |         Ethernet      |          IPv4           |            UDP         |
  |----------|-----------------------|-------------------------|------------------------|
  | SRC      |   17:17:17:17:17:17   | 1.1.1.2                 |          4097          |
  | DST      |   18:18:18:18:18:18   | 20.20.0.0               |  NAT port(1024-65535)  |

  >NOTE: The ranges of fields are circular

  >HINT: Better to keep timestamps with zero values.
  Use tool `editcap` to reset timestamps to zero.
  E.g. `editcap -S -0  generated.pcap zero_ts.pcap`

  2. Replay this PCAP file with any replay tool to **port 1 Node 2**

  ```bash
  /opt/natatech3/bin/pktgen -f PCAP_NAT/NAT_dl_1M_128b.pcap -p 1 -n 0 -g 0
  ```

  Expected result is decapsulated packets without GTPv1 header transmitted from port 1 Node 2

  |          |         Ethernet      |          IPv4                |   UDP  |      GTPv1U     |        inner   IPv4               |   inner UDP     |
  |----------|-----------------------|------------------------------|--------|-----------------|-----------------------------------|-----------------|
  | SRC      |   16:16:16:16:16:16   | 10.10.10.2                   | 2152   |  Look in table  |    192.168.0.2 - 255.255.255.255  |      4046       |
  | DST      |   1a:1a:1a:1a:1a:1a   | 10.10.10.3                   | 2152   |  GTPv1u fields  |    1.1.1.2                        |      4097       |

#### 4.2.3.5. IPv6 Downlink test case with NAT

  *IPv6 NAT is not supported yet*

# 5. RTE flow configuration

This section explains relation between groups for RTE flows.

`Group 0` - filter group (key match group) is responsible for packet filtering

`Groups [1..32]` - flow groups (flow match group)
are responsible for flows processing

5.1. Flow filter ( `Group 0` )

Port 0 filters all packets with structure:

```scapy
Eth / IPv4 / UDP
```

AND

```scapy
Eth / IPv6 / UDP
```

Filter header values:

  IPv4:

  |          |         Ethernet      |           IPv4           |      UDP        |
  |----------|-----------------------|--------------------------|-----------------|
  | SRC      |         any           |         20.0.0.0         |      any        |
  | DST      |         any           |         21.0.0.0         |      2152       |
  | SRC mask |   0x000000000000      |        0xff000000        |      0x0000     |
  | DST mask |   0x000000000000      |        0xff000000        |      0xffff     |

  IPv6:

  |          |       Ethernet        |        inner   IPv6                |   inner UDP     |
  |----------|-----------------------|------------------------------------|-----------------|
  | SRC      |   16:16:16:16:16:16   |        5555::  -  5555::00ff       |     any         |
  | DST      |   1a:1a:1a:1a:1a:1a   |        aaaa::                      |     2152        |
  | SRC mask |   0x000000000000      | 0xffff0000000000000000000000000000 |     0x0000      |
  | DST mask |   0x000000000000      | 0xffff0000000000000000000000000000 |     0xffff      |

If match check is successful, then the tunnel for GTPv1U is created in `groups 1, 2, 3 and 4`

  Groups 1 - IPv4 uplink
  Groups 2 - IPv6 uplink
  Groups 3 - IPv4 downlink
  Groups 4 - IPv4 downlink
