#include <rte_ethdev.h>
#include <rte_flow.h>

#include "filter.h"


size_t decap_ipv6_size = 0;
size_t encap_ipv6_size = 0;
uint8_t decap_ipv6_buf[MAX_DECAP_SIZE];
uint8_t encap_ipv6_buf[MAX_ENCAP_SIZE];


const uint8_t N3_IF_IPV6[16] = "\xAA\xAA\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xBB\xBB";
const uint8_t N3_IPV6_PEER[16] = "\x11\x11\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x22\x22";

const uint8_t N6_IF_IPV6[16] = "\xCC\xCC\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00";
const uint8_t IF_IPV6_MASK[16] = "\xFF\xFF\x00\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x00\x00\00";

const uint8_t IPv6_addr_any[16] = {0};

struct rte_flow* setup_tunnel_IPv6_uplink_flow(uint8_t port,
  const uint8_t match_ipv6_src[16],
  const uint8_t match_ipv6_dst[16],
  uint16_t match_udp_src,
  uint16_t match_udp_dst,
  uint32_t match_gtp_teid,
  uint8_t dscp_tag,
  __attribute__((__unused__)) bool do_NAT,
  __attribute__((__unused__)) uint16_t NAT_udp_src,
  uint8_t action_out_port_id,
  uint32_t flow_match_group,
  uint32_t *meter_id_arr, int size)
{

  struct rte_flow_attr attr;
  struct rte_flow_error error;

  struct rte_flow_item patterns[16];
  struct rte_flow_action actions[16];

  struct rte_flow *flow_filter = NULL;
  int action_it = 0;
  int pattern_it = 0;

  memset(&attr, 0x0, sizeof(attr));
  memset(&error, 0x0, sizeof(error));

  attr.group = flow_match_group;

  // Pattern: Ethernet with mask 0
  struct rte_flow_item_eth eth_spec;
  struct rte_flow_item_eth eth_mask;

  memset(&eth_spec, 0x0, sizeof(eth_spec));
  memset(&eth_mask, 0x0, sizeof(eth_mask));

  patterns[pattern_it].type = RTE_FLOW_ITEM_TYPE_ETH;
  patterns[pattern_it].spec = &eth_spec;
  patterns[pattern_it].mask = &eth_mask;
  patterns[pattern_it].last = NULL;
  pattern_it += 1;

  // Pattern: IPv6
  struct rte_flow_item_ipv6 ipv6_spec;
  struct rte_flow_item_ipv6 ipv6_mask;

  memset(&ipv6_spec, 0x0, sizeof(ipv6_spec));
  memset(&ipv6_mask, 0x0, sizeof(ipv6_mask));

  memcpy(ipv6_spec.hdr.src_addr, match_ipv6_src, 16);
  memcpy(ipv6_spec.hdr.dst_addr, match_ipv6_dst, 16);

  memset(ipv6_mask.hdr.src_addr, 0xff, 16);
  memset(ipv6_mask.hdr.dst_addr, 0xff, 16);

  patterns[pattern_it].type = RTE_FLOW_ITEM_TYPE_IPV6;
  patterns[pattern_it].spec = &ipv6_spec;
  patterns[pattern_it].mask = &ipv6_mask;
  patterns[pattern_it].last = NULL;
  pattern_it += 1;

  // Pattern: UDP
  struct rte_flow_item_udp udp_spec;
  struct rte_flow_item_udp udp_mask;

  memset(&udp_spec, 0x0, sizeof(udp_spec));
  memset(&udp_mask, 0x0, sizeof(udp_mask));

  udp_spec.hdr.src_port = match_udp_src;
  udp_spec.hdr.dst_port = match_udp_dst;

  udp_mask.hdr.src_port = RTE_BE16(0xffff);
  udp_mask.hdr.dst_port = RTE_BE16(0xffff);

  patterns[pattern_it].type = RTE_FLOW_ITEM_TYPE_UDP;
  patterns[pattern_it].spec = &udp_spec;
  patterns[pattern_it].mask = &udp_mask;
  patterns[pattern_it].last = NULL;
  pattern_it += 1;

  // Pattern: GTP teid
  struct rte_flow_item_gtp gtp_spec;
  struct rte_flow_item_gtp gtp_mask;

  memset(&gtp_spec, 0x0, sizeof(gtp_spec));
  memset(&gtp_mask, 0x0, sizeof(gtp_mask));

  gtp_spec.teid = match_gtp_teid;
  gtp_mask.teid = RTE_BE32(0xffffffff);

  patterns[pattern_it].type = RTE_FLOW_ITEM_TYPE_GTP;
  patterns[pattern_it].spec = &gtp_spec;
  patterns[pattern_it].mask = &gtp_mask;
  patterns[pattern_it].last = NULL;
  pattern_it += 1;

  // Pattern: End
  patterns[pattern_it].type = RTE_FLOW_ITEM_TYPE_END;
  patterns[pattern_it].spec = NULL;
  patterns[pattern_it].mask = NULL;
  patterns[pattern_it].last = NULL;
  pattern_it += 1;

  // Action: Meter
  struct rte_flow_action_meter meter[16] = {0};
  if (meter_id_arr) {
    for (int i = 0; i < size; i++) {
      meter[i].mtr_id =  meter_id_arr[i];

      actions[action_it].type = RTE_FLOW_ACTION_TYPE_METER;
      actions[action_it].conf = &meter[i];
      action_it += 1;
    }
  }

  // Action: Decap
  struct rte_flow_action_raw_decap raw_decap;

  raw_decap.data = encap_ipv6_buf;
  raw_decap.size = encap_ipv6_size;

  actions[action_it].type = RTE_FLOW_ACTION_TYPE_RAW_DECAP;
  actions[action_it].conf = &raw_decap;
  action_it += 1;

  // Action: Encap
  struct rte_flow_action_raw_encap raw_encap;

  raw_encap.data = decap_ipv6_buf;
  raw_encap.size = decap_ipv6_size;
  raw_encap.preserve = NULL;

  actions[action_it].type = RTE_FLOW_ACTION_TYPE_RAW_ENCAP;
  actions[action_it].conf = &raw_encap;
  action_it += 1;

  // Action: Modify field IPv6 dscp
  struct rte_flow_action_modify_field modify_ipv6_dscp;
  memset(&modify_ipv6_dscp, 0x0, sizeof(modify_ipv6_dscp));

  modify_ipv6_dscp.operation = RTE_FLOW_MODIFY_SET;
  modify_ipv6_dscp.width = 1;
  modify_ipv6_dscp.dst.field = RTE_FLOW_FIELD_IPV6_DSCP;
  modify_ipv6_dscp.src.field = RTE_FLOW_FIELD_VALUE;

  modify_ipv6_dscp.src.value[0] = dscp_tag;

  actions[action_it].type = RTE_FLOW_ACTION_TYPE_MODIFY_FIELD;
  actions[action_it].conf = &modify_ipv6_dscp;
  action_it += 1;

  // struct rte_flow_action_modify_field modify_ipv6_src;
  // struct rte_flow_action_modify_field modify_upd_src;
  // if (do_NAT) {
  //   // Action: Modify field IPv6 src
  //   memset(&modify_ipv6_src, 0x0, sizeof(modify_ipv6_src));

  //   modify_ipv6_src.operation = RTE_FLOW_MODIFY_SET;
  //   modify_ipv6_src.width = 16;
  //   modify_ipv6_src.src.field = RTE_FLOW_FIELD_IPV6_SRC;
  //   modify_ipv6_src.dst.level = 1; // set encap level to modify outer packet headers
  //   modify_ipv6_src.src.field = RTE_FLOW_FIELD_VALUE;

  //   memcpy(modify_ipv6_src.src.value, N6_ipv6_addr, 16);

  //   actions[action_it].type = RTE_FLOW_ACTION_TYPE_MODIFY_FIELD;
  //   actions[action_it].conf = &modify_ipv6_src;
  //   action_it += 1;

  //     // Action: Modify field UDP src port ----> NAT
  //   memset(&modify_upd_src, 0x0, sizeof(modify_upd_src));

  //   modify_upd_src.operation = RTE_FLOW_MODIFY_SET;
  //   modify_upd_src.width = 2;
  //   modify_upd_src.dst.field = RTE_FLOW_FIELD_UDP_PORT_SRC;
  //   modify_upd_src.dst.level = 1; // set encap level to modify outer packet headers
  //   modify_upd_src.src.field = RTE_FLOW_FIELD_VALUE;

  //   modify_upd_src.src.value[0] = (NAT_udp_src & (uint32_t)0x00ff);
  //   modify_upd_src.src.value[1] = (NAT_udp_src & (uint32_t)0xff00) >> 8;

  //   actions[action_it].type = RTE_FLOW_ACTION_TYPE_MODIFY_FIELD;
  //   actions[action_it].conf = &modify_upd_src;
  //   action_it += 1;
  // }

  // Action: Port id
  struct rte_flow_action_port_id port_id;

  port_id.original = 0;
  port_id.reserved = 0;
  port_id.id = action_out_port_id;

  actions[action_it].type = RTE_FLOW_ACTION_TYPE_PORT_ID;
  actions[action_it].conf = &port_id;
  action_it += 1;

  // Action: End
  actions[action_it].type = RTE_FLOW_ACTION_TYPE_END;
  actions[action_it].conf = NULL;
  action_it += 1;

  flow_filter = rte_flow_create(port, &attr, patterns, actions, &error);

  if (!flow_filter) {
    printf("Error: Cannot create tunnel UpLink IPv6 flow: %s\n", error.message);
  }

  return flow_filter;
}

struct rte_flow* setup_tunnel_IPv6_downlink_flow(uint8_t port,
  const uint8_t match_ipv6_src[16],
  const uint8_t match_ipv6_dst[16],
  uint16_t match_udp_src,
  uint16_t match_udp_dst,
   __attribute__((__unused__)) const uint8_t UE_ipv6[16],
   __attribute__((__unused__)) uint16_t NAT_udp_src,
  uint32_t encap_gtp_teid,
  uint8_t qfi,
  __attribute__((__unused__)) bool do_NAT,
  uint8_t action_out_port_id,
  uint32_t flow_match_group,
  uint32_t *meter_id_arr, int size)
{

  struct rte_flow_attr attr;
  struct rte_flow_error error;

  struct rte_flow_item patterns[16];
  struct rte_flow_action actions[16];

  struct rte_flow *flow_filter = NULL;
  int action_it = 0;
  int pattern_it = 0;

  memset(&attr, 0x0, sizeof(attr));
  memset(&error, 0x0, sizeof(error));

  attr.group = flow_match_group;

  // Pattern: Ethernet with mask 0
  struct rte_flow_item_eth eth_spec;
  struct rte_flow_item_eth eth_mask;

  memset(&eth_spec, 0x0, sizeof(eth_spec));
  memset(&eth_mask, 0x0, sizeof(eth_mask));

  patterns[pattern_it].type = RTE_FLOW_ITEM_TYPE_ETH;
  patterns[pattern_it].spec = &eth_spec;
  patterns[pattern_it].mask = &eth_mask;
  patterns[pattern_it].last = NULL;
  pattern_it += 1;

  // Pattern: IPv6
  struct rte_flow_item_ipv6 ipv6_spec;
  struct rte_flow_item_ipv6 ipv6_mask;

  memset(&ipv6_spec, 0x0, sizeof(ipv6_spec));
  memset(&ipv6_mask, 0x0, sizeof(ipv6_mask));

  memcpy(ipv6_spec.hdr.src_addr, match_ipv6_src, 16);
  memcpy(ipv6_spec.hdr.dst_addr, match_ipv6_dst, 16);

  memset(ipv6_mask.hdr.src_addr, 0xff, 16);
  memset(ipv6_mask.hdr.dst_addr, 0xff, 16);

  patterns[pattern_it].type = RTE_FLOW_ITEM_TYPE_IPV6;
  patterns[pattern_it].spec = &ipv6_spec;
  patterns[pattern_it].mask = &ipv6_mask;
  patterns[pattern_it].last = NULL;
  pattern_it += 1;

  // Pattern: UDP
  struct rte_flow_item_udp udp_spec;
  struct rte_flow_item_udp udp_mask;

  memset(&udp_spec, 0x0, sizeof(udp_spec));
  memset(&udp_mask, 0x0, sizeof(udp_mask));

  // if (do_NAT) // ----> NAT
  // {
  //   udp_spec.hdr.src_port = match_udp_dst;
  //   udp_spec.hdr.dst_port = RTE_BE16(NAT_udp_src);
  // }
  // else {
    udp_spec.hdr.src_port = match_udp_src;
    udp_spec.hdr.dst_port = match_udp_dst;
  // }

  udp_mask.hdr.src_port = RTE_BE16(0xffff);
  udp_mask.hdr.dst_port = RTE_BE16(0xffff);

  patterns[pattern_it].type = RTE_FLOW_ITEM_TYPE_UDP;
  patterns[pattern_it].spec = &udp_spec;
  patterns[pattern_it].mask = &udp_mask;
  patterns[pattern_it].last = NULL;
  pattern_it += 1;

  // Pattern: End
  patterns[pattern_it].type = RTE_FLOW_ITEM_TYPE_END;
  patterns[pattern_it].spec = NULL;
  patterns[pattern_it].mask = NULL;
  patterns[pattern_it].last = NULL;
  pattern_it += 1;

  // Action: Meter
  struct rte_flow_action_meter meter[16] = {0};
  if (meter_id_arr) {
    for (int i = 0; i < size; i++) {
      meter[i].mtr_id =  meter_id_arr[i];

      actions[action_it].type = RTE_FLOW_ACTION_TYPE_METER;
      actions[action_it].conf = &meter[i];
      action_it += 1;
    }
  }

  // Action: Decap
  struct rte_flow_action_raw_decap raw_decap;

  raw_decap.data = decap_ipv6_buf;
  raw_decap.size = decap_ipv6_size;

  actions[action_it].type = RTE_FLOW_ACTION_TYPE_RAW_DECAP;
  actions[action_it].conf = &raw_decap;
  action_it += 1;

  // Action: Encap
  struct rte_flow_action_raw_encap raw_encap;

  raw_encap.data = encap_ipv6_buf;
  raw_encap.size = encap_ipv6_size;
  raw_encap.preserve = NULL;

  actions[action_it].type = RTE_FLOW_ACTION_TYPE_RAW_ENCAP;
  actions[action_it].conf = &raw_encap;
  action_it += 1;

  // Action: Modify field IPv6 dscp
  struct rte_flow_action_modify_field modify_ipv6_dscp;
  memset(&modify_ipv6_dscp, 0x0, sizeof(modify_ipv6_dscp));

  modify_ipv6_dscp.operation = RTE_FLOW_MODIFY_SET;
  modify_ipv6_dscp.width = 1;
  modify_ipv6_dscp.dst.field = RTE_FLOW_FIELD_IPV6_DSCP;
  modify_ipv6_dscp.src.field = RTE_FLOW_FIELD_VALUE;

  modify_ipv6_dscp.src.value[0] = DEFAULT_DSCP;

  actions[action_it].type = RTE_FLOW_ACTION_TYPE_MODIFY_FIELD;
  actions[action_it].conf = &modify_ipv6_dscp;
  action_it += 1;

  //   struct rte_flow_action_modify_field modify_ipv4_inner_dst;
  //   struct rte_flow_action_modify_field modify_upd_dst;
  //   if (do_NAT) {
  //     // Action: Modify field IPv6 dst ----> NAT
  //   memset(&modify_ipv4_inner_dst, 0x0, sizeof(modify_ipv4_inner_dst));

  //   modify_ipv4_inner_dst.operation = RTE_FLOW_MODIFY_SET;
  //   modify_ipv4_inner_dst.width = 16;
  //   modify_ipv4_inner_dst.dst.field = RTE_FLOW_FIELD_IPV6_DST;
  //   modify_ipv4_inner_dst.dst.level = 2; // set encap level to modify inner packet headers
  //   modify_ipv4_inner_dst.src.field = RTE_FLOW_FIELD_VALUE;

  //   memcpy(modify_ipv4_inner_dst.src.value, UE_ipv6, 16);

  //   actions[action_it].type = RTE_FLOW_ACTION_TYPE_MODIFY_FIELD;
  //   actions[action_it].conf = &modify_ipv4_inner_dst;
  //   action_it += 1;

  //   // Action: Modify field UDP dst port ----> NAT
  //   memset(&modify_upd_dst, 0x0, sizeof(modify_upd_dst));

  //   modify_upd_dst.operation = RTE_FLOW_MODIFY_SET;
  //   modify_upd_dst.width = 2;
  //   modify_upd_dst.dst.field = RTE_FLOW_FIELD_UDP_PORT_DST;
  //   modify_upd_dst.dst.level = 2; // set encap level to modify inner packet headers
  //   modify_upd_dst.src.field = RTE_FLOW_FIELD_VALUE;

  //   modify_upd_dst.src.value[0] = (match_udp_src & (uint32_t)0x00ff);
  //   modify_upd_dst.src.value[1] = (match_udp_src & (uint32_t)0xff00) >> 8;

  //   actions[action_it].type = RTE_FLOW_ACTION_TYPE_MODIFY_FIELD;
  //   actions[action_it].conf = &modify_upd_dst;
  //   action_it += 1;
  // }

    // Action: Modify field GTP TEID
  struct rte_flow_action_modify_field modify_gtp_teid;
  memset(&modify_gtp_teid, 0x0, sizeof(modify_gtp_teid));

  modify_gtp_teid.operation = RTE_FLOW_MODIFY_SET;
  modify_gtp_teid.width = 4;
  modify_gtp_teid.dst.field = RTE_FLOW_FIELD_GTP_TEID;
  modify_gtp_teid.src.field = RTE_FLOW_FIELD_VALUE;

  modify_gtp_teid.src.value[0] = (encap_gtp_teid & (uint32_t)0x000000ff);
  modify_gtp_teid.src.value[1] = (encap_gtp_teid & (uint32_t)0x0000ff00) >> 8;
  modify_gtp_teid.src.value[2] = (encap_gtp_teid & (uint32_t)0x00ff0000) >> 16;
  modify_gtp_teid.src.value[3] = (encap_gtp_teid & (uint32_t)0xff000000) >> 24;

  actions[action_it].type = RTE_FLOW_ACTION_TYPE_MODIFY_FIELD;
  actions[action_it].conf = &modify_gtp_teid;
  action_it += 1;

  // Action: Modify field GTP psc QFI
  struct rte_flow_action_modify_field modify_gtp_qfi;
  memset(&modify_gtp_qfi, 0x0, sizeof(modify_gtp_qfi));

  modify_gtp_qfi.operation = RTE_FLOW_MODIFY_SET;
  modify_gtp_qfi.width = 1;
  modify_gtp_qfi.dst.field = RTE_FLOW_FIELD_GTP_PSC_QFI;
  modify_gtp_qfi.src.field = RTE_FLOW_FIELD_VALUE;

  modify_gtp_qfi.src.value[0] = qfi;

  actions[action_it].type = RTE_FLOW_ACTION_TYPE_MODIFY_FIELD;
  actions[action_it].conf = &modify_gtp_qfi;
  action_it += 1;

  // Action: Port id
  struct rte_flow_action_port_id port_id;

  port_id.original = 0;
  port_id.reserved = 0;
  port_id.id = action_out_port_id;

  actions[action_it].type = RTE_FLOW_ACTION_TYPE_PORT_ID;
  actions[action_it].conf = &port_id;
  action_it += 1;

  // Action: End
  actions[action_it].type = RTE_FLOW_ACTION_TYPE_END;
  actions[action_it].conf = NULL;
  action_it += 1;

  flow_filter = rte_flow_create(port, &attr, patterns, actions, &error);

  if (!flow_filter) {
    printf("Error: Cannot create tunnel DownLink IPv6 flow: %s\n", error.message);
  }

  return flow_filter;
}

struct rte_flow* setup_main_uplink_IPv6_filter(uint8_t port,
  const uint8_t match_ipv6_src[16],
  const uint8_t match_ipv6_src_mask[16],
  const uint8_t match_ipv6_dst[16],
  const uint8_t match_ipv6_dst_mask[16],
  uint16_t* queues,
  uint32_t queue_num,
  uint32_t flow_match_group)
{
  struct rte_flow_attr attr;
  struct rte_flow_error error;

  struct rte_flow_item patterns[16];
  struct rte_flow_action actions[16];

  struct rte_flow *flow_filter = NULL;
  int action_it = 0;
  int pattern_it = 0;

  memset(&attr, 0x0, sizeof(attr));
  memset(&error, 0x0, sizeof(error));

  attr.group = KEY_MATCH_GROUP;

  // Pattern: Ethernet with mask 0
  struct rte_flow_item_eth eth_spec;
  struct rte_flow_item_eth eth_mask;

  memset(&eth_spec, 0x0, sizeof(eth_spec));
  memset(&eth_mask, 0x0, sizeof(eth_mask));

  patterns[pattern_it].type = RTE_FLOW_ITEM_TYPE_ETH;
  patterns[pattern_it].spec = &eth_spec;
  patterns[pattern_it].mask = &eth_mask;
  patterns[pattern_it].last = NULL;
  pattern_it += 1;

  // Pattern: IPv6
  struct rte_flow_item_ipv6 ipv6_spec;
  struct rte_flow_item_ipv6 ipv6_mask;

  memset(&ipv6_spec, 0x0, sizeof(ipv6_spec));
  memset(&ipv6_mask, 0x0, sizeof(ipv6_mask));

  memcpy(ipv6_spec.hdr.src_addr, match_ipv6_src, 16);
  memcpy(ipv6_spec.hdr.dst_addr, match_ipv6_dst, 16);

  memcpy(ipv6_mask.hdr.src_addr, match_ipv6_src_mask, 16);
  memcpy(ipv6_mask.hdr.dst_addr, match_ipv6_dst_mask, 16);

  patterns[pattern_it].type = RTE_FLOW_ITEM_TYPE_IPV6;
  patterns[pattern_it].spec = &ipv6_spec;
  patterns[pattern_it].mask = &ipv6_mask;
  patterns[pattern_it].last = NULL;
  pattern_it += 1;

  // Pattern: UDP
  struct rte_flow_item_udp udp_spec;
  struct rte_flow_item_udp udp_mask;

  memset(&udp_spec, 0x0, sizeof(udp_spec));
  memset(&udp_mask, 0x0, sizeof(udp_mask));

  udp_spec.hdr.src_port = RTE_BE16(0);
  udp_spec.hdr.dst_port = RTE_BE16(2152);

  udp_mask.hdr.src_port = RTE_BE16(0x0000);
  udp_mask.hdr.dst_port = RTE_BE16(0xffff);

  patterns[pattern_it].type = RTE_FLOW_ITEM_TYPE_UDP;
  patterns[pattern_it].spec = &udp_spec;
  patterns[pattern_it].mask = &udp_mask;
  patterns[pattern_it].last = NULL;
  pattern_it += 1;

  // Pattern: GTP
  struct rte_flow_item_gtp gtp_spec;
  struct rte_flow_item_gtp gtp_mask;

  memset(&gtp_spec, 0x0, sizeof(gtp_spec));
  memset(&gtp_mask, 0x0, sizeof(gtp_mask));

  patterns[pattern_it].type = RTE_FLOW_ITEM_TYPE_GTP;
  patterns[pattern_it].spec = &gtp_spec;
  patterns[pattern_it].mask = &gtp_mask;
  patterns[pattern_it].last = NULL;
  pattern_it += 1;

  // Pattern: End
  patterns[pattern_it].type = RTE_FLOW_ITEM_TYPE_END;
  patterns[pattern_it].spec = NULL;
  patterns[pattern_it].mask = NULL;
  patterns[pattern_it].last = NULL;
  pattern_it += 1;

  // Action: RSS
  struct rte_flow_action_rss rss;

  rss.queue_num = queue_num;
  rss.queue = queues;
  rss.func = RTE_ETH_HASH_FUNCTION_DEFAULT;
  rss.types = RTE_ETH_RSS_UDP;
  rss.level = 2;  // Inner packet
  rss.key = NULL;
  rss.key_len = 0;

  actions[action_it].type = RTE_FLOW_ACTION_TYPE_RSS;
  actions[action_it].conf = &rss;
  action_it += 1;

  // Action: Jump
  struct rte_flow_action_jump jump;

  jump.group = flow_match_group;

  actions[action_it].type = RTE_FLOW_ACTION_TYPE_JUMP;
  actions[action_it].conf = &jump;
  action_it += 1;

  // Action: End
  actions[action_it].type = RTE_FLOW_ACTION_TYPE_END;
  actions[action_it].conf = NULL;
  action_it += 1;

  flow_filter = rte_flow_create(port, &attr, patterns, actions, &error);

  if (!flow_filter) {
    printf("Error: Cannot create main UpLink IPv6 filter: %s\n", error.message);
  }

  return flow_filter;
}

struct rte_flow* setup_main_downlink_IPv6_filter(uint8_t port,
  const uint8_t match_ipv6_src[16],
  const uint8_t match_ipv6_src_mask[16],
  const uint8_t match_ipv6_dst[16],
  const uint8_t match_ipv6_dst_mask[16],
  uint32_t flow_match_group)
{
  struct rte_flow_attr attr;
  struct rte_flow_error error;

  struct rte_flow_item patterns[16];
  struct rte_flow_action actions[16];

  struct rte_flow *flow_filter = NULL;
  int action_it = 0;
  int pattern_it = 0;

  memset(&attr, 0x0, sizeof(attr));
  memset(&error, 0x0, sizeof(error));

  attr.group = KEY_MATCH_GROUP;

  // Pattern: Ethernet with mask 0
  struct rte_flow_item_eth eth_spec;
  struct rte_flow_item_eth eth_mask;

  memset(&eth_spec, 0x0, sizeof(eth_spec));
  memset(&eth_mask, 0x0, sizeof(eth_mask));

  patterns[pattern_it].type = RTE_FLOW_ITEM_TYPE_ETH;
  patterns[pattern_it].spec = &eth_spec;
  patterns[pattern_it].mask = &eth_mask;
  patterns[pattern_it].last = NULL;
  pattern_it += 1;

  // Pattern: IPv6
  struct rte_flow_item_ipv6 ipv6_spec;
  struct rte_flow_item_ipv6 ipv6_mask;

  memset(&ipv6_spec, 0x0, sizeof(ipv6_spec));
  memset(&ipv6_mask, 0x0, sizeof(ipv6_mask));

  memcpy(ipv6_spec.hdr.src_addr, match_ipv6_src, 16);
  memcpy(ipv6_spec.hdr.dst_addr, match_ipv6_dst, 16);

  memcpy(ipv6_mask.hdr.src_addr, match_ipv6_src_mask, 16);
  memcpy(ipv6_mask.hdr.dst_addr, match_ipv6_dst_mask, 16);

  patterns[pattern_it].type = RTE_FLOW_ITEM_TYPE_IPV6;
  patterns[pattern_it].spec = &ipv6_spec;
  patterns[pattern_it].mask = &ipv6_mask;
  patterns[pattern_it].last = NULL;
  pattern_it += 1;

  // Pattern: UDP
  struct rte_flow_item_udp udp_spec;
  struct rte_flow_item_udp udp_mask;

  memset(&udp_spec, 0x0, sizeof(udp_spec));
  memset(&udp_mask, 0x0, sizeof(udp_mask));

  patterns[pattern_it].type = RTE_FLOW_ITEM_TYPE_UDP;
  patterns[pattern_it].spec = &udp_spec;
  patterns[pattern_it].mask = &udp_mask;
  patterns[pattern_it].last = NULL;
  pattern_it += 1;

  // Pattern: End
  patterns[pattern_it].type = RTE_FLOW_ITEM_TYPE_END;
  patterns[pattern_it].spec = NULL;
  patterns[pattern_it].mask = NULL;
  patterns[pattern_it].last = NULL;
  pattern_it += 1;

  // Action: Jump
  struct rte_flow_action_jump jump;

  jump.group = flow_match_group;

  actions[action_it].type = RTE_FLOW_ACTION_TYPE_JUMP;
  actions[action_it].conf = &jump;
  action_it += 1;

  // Action: End
  actions[action_it].type = RTE_FLOW_ACTION_TYPE_END;
  actions[action_it].conf = NULL;
  action_it += 1;

  flow_filter = rte_flow_create(port, &attr, patterns, actions, &error);

  if (!flow_filter) {
    printf("Error: Cannot create main DownLink IPv6 filter: %s\n", error.message);
  }

  return flow_filter;
}
