# Napatech Kubernetes demo

This package contains Napatech inline demo applications, Napatech Device Plugin for Kubernetes and set of Dockerfile
and manifest examples for easier demo application deployment and testing.


## Directory structure

```
kubernetes_v22.11.1
|-- README.md                   # This documentation
|-- docker/                     # Definition of container images and start scripts for NT demo applications
|-- manifest/                   # Definition of manifests for deployment of NT demo applications
`-- ntdevplugin/                # Napatech Device Plugin (NTDevPlugin) for Kubernetes
```

## Build of container images

NOTE: It is expected that all commands are executed from the root directory of the release package.

### Napatech Device Plugin

Please follow NTDevPlugin documentation [here](ntdevplugin/README.md).

### Inline demo applications

Provided docker file examples can be used to generate a local container images with several inline demo
applications. It is possible to use `docker`, `podman` or `nerdctl` to build container images. Please select
a build method, which suits your Kubernetes configuration, especially chosen container runtime.

Example of build of images:
```bash
docker build -t k8s.io/ntapps:1.1 -f kubernetes_v22.11.1/docker/Dockerfile.ntapps .
docker build -t k8s.io/ntforward:1.1 -f kubernetes_v22.11.1/docker/Dockerfile.ntforward .
```

### Inline-forward Demo

Figure with simplified k8s demo configuration with NT Main Application container deployed
at NTDevPlugin POD.

```
    ........................................             ......................
    :            DUT / K8S node            :             :       TGEN         :
    : +---------------+  +---------------+ :             :                    :
    : | forward app   |  | forward app   | :             :                    :
    : | 2 VFs forward |  | 2 VFs forward | :             : Generating VLAN    :
    : | vlan 44,45    |  | vlan 46,47    | :             : tagged traffic     :
    : | (virtio PMD)  |  | (virtio PMD)  | :             : VLAN 44,45,46,47.. :
    : +---------------+  +---------------+ :             :                    :
    :         ^            ^               :             :                    :
    :         : provides:  : provides:     :             :                    :
    :         : nt_vf:2    : nt_vf:2       :             :                    :
    :         :............:               :             :                    :
    :         :                            :             :                    :
    : +-------------------+                :             :                    :
    : |  NTDevPlugin POD  |                :             :                    :
    : |  +-------------+  |                :             :                    :
    : |  |    init     |  |                :             :                    :
    : |  +-------------+  |                :             :                    :
    : |  +-------------+  |                :             :                    :
    : |  | NTDevPlugin |  |                :             :                    :
    : |  +-------------+  |                :             :                    :
    : |  +-------------+  |       +--------:             :--------+           :
    : |  | NT Main App |<-|------>| NT NIC :<=== LAN ===>: NT NIC |           :
    : |  +-------------+  |       +--------:             :--------+           :
    : +-------------------+                :             :                    :
    :......................................:             :....................:
```

Following NTDevPlugin ConfigMap can be used to configure depicted demo setup:
```yaml
apiVersion: v1
kind: ConfigMap
metadata:
  name: napatech-device-plugin-config
  namespace: kube-system
data:
  NicPci: "autodetect"
  NicAutoInit: "enabled"
  MainApp: "enabled"
  vfCount: "4"
  vfMacStart: "02:11:22:33:44:00"
  vfVlanStart: "44"
  vfNumberOfQueues: "1"
immutable: true
```

Please note following configuraiton values:
* vfCount: "4" - demo setup requires at least 4 VFs to be configured
* MainApp: "enabled" - NT Main application will be started automatically as part of NTDevPlugin POD
* vfNumberOfQueues: "1" - by default 1 queue is used per VF; Number of queues can be increased to
  fine tune performance

Following manifest examples can be used to deploy NTDevPlugin and two instances of `forward` application:
```bash
kubectl apply -f kubernetes_v22.11.1/ntdevplugin/ntdevplugin.yaml
kubectl apply -f kubernetes_v22.11.1/manifest/k8s-forward.yaml
```

Demo applications can be removed by:
```bash
kubectl delete -f kubernetes_v22.11.1/manifest/k8s-forward.yaml
kubectl delete -f kubernetes_v22.11.1/ntdevplugin/ntdevplugin.yaml
```

### Inline UPF Demo

Figure with simplified k8s demo configuration:

```
    .................................           ..................
    :        DUT / K8S node         :           :      TGEN      :
    :  +-----------------+          :           :                :
    :  | inline_upf_demo |          :           :                :
    :  | app with NT PF  |<------+  :           : Generating UPF :
    :  +-----------------+       |  :           :    Traffic     :
    :      ^ provides:           |  :           :                :
    :      : nt_pf:1             |  :           :                :
    :  +-----------------+       |  :           :                :
    :  | NTDevPlugin POD |       |  :           :                :
    :  | +-------------+ |       |  :           :                :
    :  | |    init     | |       |  :           :                :
    :  | +-------------+ |       v  :           :                :
    :  | +-------------+ | +--------:           :--------+       :
    :  | | NTDevPlugin | | | NT NIC :<== LAN ==>: NT NIC |       :
    :  | +-------------+ | +--------:           :--------+       :
    :  +-----------------+          :           :                :
    :...............................:           :................:
```

Following NTDevPlugin ConfigMap can be used to configure depicted demo setup:
```yaml
apiVersion: v1
kind: ConfigMap
metadata:
  name: napatech-device-plugin-config
  namespace: kube-system
data:
  NicPci: "autodetect"
  NicAutoInit: "enabled"
  MainApp: "enabled"
  vfCount: "4"
  vfMacStart: "02:11:22:33:44:00"
  vfVlanStart: "44"
  vfNumberOfQueues: "1"
immutable: true
```
Please note following configuration value:
* MainApp: "disabled" - NT Main application will be disabled, so `inline_upf_demo` can request
  `nt_pf` resource from NTDevPlugin

Following manifest examples can be used to deploy `inline_upf_demo` application and NTDevPlugin:
```bash
kubectl apply -f kubernetes_v22.11.1/ntdevplugin/ntdevplugin.yaml
kubectl apply -f kubernetes_v22.11.1/manifest/k8s-inline-upf-demo.yaml
```

Demo setup can be removed by:
```bash
kubectl delete -f kubernetes_v22.11.1/manifest/k8s-inline-upf-demo.yaml
kubectl delete -f kubernetes_v22.11.1/ntdevplugin/ntdevplugin.yaml
```

### Napatech Tools

There are three Napatech tools `monitoring`, `statistic` and `sensors` available inside `ntapps` and `ntmain` container images.
These tools can be used to monitor and debug demo setups described above.

NOTE: Following examples of tools execution expects, that NTDevPlugin is deployed and `MainApp` is enabled at `napatech-device-plugin-config`.

Example of interactive execution of `monitoring` app:
```bash
kubectl -n kube-system exec -it daemonsets/napatech-device-plugin -c napatech-main-app -- monitoring
```
Example of `statistic` executions:
```bash
kubectl -n kube-system exec daemonsets/napatech-device-plugin -c napatech-main-app -- statistic --help
kubectl -n kube-system exec daemonsets/napatech-device-plugin -c napatech-main-app -- statistic -g flm_stats
```
Example of `sensors` execution:
```bash
kubectl -n kube-system exec daemonsets/napatech-device-plugin -c napatech-main-app -- sensors
```
